@@ -275,7 +275,7 @@ cor_errorsar <- function(W) {
 #'   implemented based on the case study of Max Joseph
 #'   (\url{https://github.com/mbjoseph/CARstan}). The \code{icar} and 
 #'   \code{bym2} type is implemented based on the case study of Mitzi Morris
-#'   (\url{http://mc-stan.org/users/documentation/case-studies/icar_stan.html}).
+#'   (\url{https://mc-stan.org/users/documentation/case-studies/icar_stan.html}).
 #'   
 #' @examples
 #' \dontrun{
@@ -518,7 +518,7 @@ autocor <- function(object, ...) {
 # extract variables for autocorrelation structures
 # @param autocor object of class 'cor_brms'
 # @return a list with elements 'time', and 'group'
-parse_autocor <- function(autocor) {
+terms_autocor <- function(autocor) {
   out <- list()
   formula <- autocor$formula
   if (is.null(formula)) {
@@ -560,7 +560,7 @@ as_formula_cor_brms <- function(x) {
     return(NULL)
   }
   args <- data2 <- list()
-  pac <- parse_autocor(x)
+  pac <- terms_autocor(x)
   if (is.cor_arma(x)) {
     fun <- "arma"
     args$time <- pac$time
