@@ -72,22 +72,29 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
                          data2 = NULL, autocor = NULL, cov_ranef = NULL, 
                          sample_prior = c("no", "yes", "only"), 
                          sparse = NULL, knots = NULL, stanvars = NULL,
-                         stan_funs = NULL, recompile = FALSE,
-                         combine = TRUE, fit = NA,
-                         seed = NA, file = NULL, ...) {
+                         stan_funs = NULL, silent = 1, recompile = FALSE,
+                         combine = TRUE, fit = NA, seed = NA, 
+                         file = NULL, file_refit = "never", ...) {
   
   combine <- as_one_logical(combine)
+  file_refit <- match.arg(file_refit, file_refit_options())
   if (!is.null(file)) {
+    if (file_refit == "on_change") {
+      stop2("file_refit = 'on_change' is not supported for brm_multiple yet.")
+    }
     # optionally load saved model object
     if (!combine) {
       stop2("Cannot use 'file' if 'combine' is FALSE.")
     }
-    fits <- read_brmsfit(file)
-    if (!is.null(fits)) {
-      return(fits)
+    if (file_refit != "always") {
+      fits <- read_brmsfit(file)
+      if (!is.null(fits)) {
+        return(fits)
+      } 
     }
   }
   
+  silent <- validate_silent(silent)
   recompile <- as_one_logical(recompile)
   data_name <- substitute_name(data)
   if (inherits(data, "mids")) {
@@ -112,10 +119,12 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
     args <- nlist(
       formula, data = data[[1]], family, prior, data2 = data2[[1]], 
       autocor, cov_ranef, sample_prior, sparse, knots, stanvars, 
-      stan_funs, seed, ...
+      stan_funs, silent, seed, ...
     )
     args$chains <- 0
-    message("Compiling the C++ model")
+    if (silent < 2) {
+      message("Compiling the C++ model")
+    }
     fit <- suppressMessages(do_call(brm, args))
   }
   
@@ -130,12 +139,14 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
   for (i in seq_along(data)) {
     futures[[i]] <- future::future(
       update(fit, newdata = data[[i]], data2 = data2[[i]],
-             recompile = recompile, ...),
-      packages = "brms"
+             recompile = recompile, silent = silent, ...),
+      packages = "brms", seed = TRUE
     )
   }
   for (i in seq_along(data)) {
-    message("Fitting imputed model ", i)
+    if (silent < 2) {
+      message("Fitting imputed model ", i) 
+    }
     fits[[i]] <- future::value(futures[[i]]) 
     rhats[[i]] <- data.frame(as.list(rhat(fits[[i]])))
     if (any(rhats[[i]] > 1.1, na.rm = TRUE)) {
@@ -149,7 +160,7 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
     class(fits) <- c("brmsfit_multiple", class(fits))
   }
   if (!is.null(file)) {
-    write_brmsfit(fits, file)
+    fits <- write_brmsfit(fits, file)
   }
   fits
 }
@@ -157,7 +168,7 @@ brm_multiple <- function(formula, data, family = gaussian(), prior = NULL,
 #' Combine Models fitted with \pkg{brms}
 #' 
 #' Combine multiple \code{brmsfit} objects, which fitted the same model.
-#' This is usefuly for instance when having manually run models in parallel.
+#' This is usefully for instance when having manually run models in parallel.
 #' 
 #' @param ... One or more \code{brmsfit} objects.
 #' @param mlist Optional list of one or more \code{brmsfit} objects.
@@ -186,13 +197,13 @@ combine_models <- function(..., mlist = NULL, check_data = TRUE) {
     models[[i]] <- restructure(models[[i]])
   }
   ref_formula <- formula(models[[1]])
-  ref_pars <- parnames(models[[1]])
+  ref_pars <- variables(models[[1]])
   ref_mf <- model.frame(models[[1]]) 
   for (i in seq_along(models)[-1]) {
     if (!is_equal(formula(models[[i]]), ref_formula)) {
       stop2("Models 1 and ", i, " have different formulas.")
     }
-    if (!is_equal(parnames(models[[i]]), ref_pars)) {
+    if (!is_equal(variables(models[[i]]), ref_pars)) {
       stop2("Models 1 and ", i, " have different parameters.")
     }
     if (check_data && !is_equal(model.frame(models[[i]]), ref_mf)) {
