@@ -2,14 +2,15 @@
 #' 
 #' Parse formulas objects for use in \pkg{brms}.
 #' 
+#' @aliases parse_bf
+#' 
 #' @inheritParams brm
 #' @param check_response Logical; Indicates whether the left-hand side 
 #'   of \code{formula} (i.e. response variables and addition arguments) 
 #'   should be parsed. If \code{FALSE}, \code{formula} may also be one-sided.
 #' @param resp_rhs_all Logical; Indicates whether to also include response 
 #'   variables on the right-hand side of formula \code{.$allvars}, 
-#'   where \code{.} represents the output of \code{parse_bf}.
-#' @param mv Indicates if the univariate model is part of a multivariate model.
+#'   where \code{.} represents the output of \code{brmsterms}.
 #' @param ... Further arguments passed to or from other methods.
 #'  
 #' @return An object of class \code{brmsterms} or \code{mvbrmsterms} 
@@ -31,43 +32,50 @@
 #'   \code{\link{mvbrmsformula}}
 #' 
 #' @export
-parse_bf <- function(formula, ...) {
-  UseMethod("parse_bf")
+brmsterms <- function(formula, ...) {
+  UseMethod("brmsterms")
 }
 
-#' @rdname parse_bf
+# the name 'parse_bf' is deprecated as of brms 2.12.4
+# remove it eventually in brms 3.0
 #' @export
-parse_bf.default <- function(formula, family = NULL, autocor = NULL, ...) {
-  x <- validate_formula(formula, family = family, autocor = autocor)
-  parse_bf(x, ...)
+parse_bf <- function(x, ...) {
+  warning2("Method 'parse_bf' is deprecated. Please use 'brmsterms' instead.")
+  UseMethod("brmsterms")
 }
 
-#' @rdname parse_bf
+#' @rdname brmsterms
 #' @export
-parse_bf.brmsformula <- function(formula, family = NULL, autocor = NULL, 
-                                 check_response = TRUE, resp_rhs_all = TRUE,
-                                 mv = FALSE, ...) {
-  x <- validate_formula(formula, family = family, autocor = autocor)
-  mv <- as_one_logical(mv)
+brmsterms.default <- function(formula, ...) {
+  brmsterms(validate_formula(formula), ...)
+}
+
+#' @rdname brmsterms
+#' @export
+brmsterms.brmsformula <- function(formula, check_response = TRUE, 
+                                  resp_rhs_all = TRUE, ...) {
+  x <- validate_formula(formula)
+  mv <- isTRUE(x$mv)
   rescor <- mv && isTRUE(x$rescor)
   mecor <- isTRUE(x$mecor)
   formula <- x$formula
   family <- x$family
-  y <- nlist(formula, family, mv, rescor, mecor) 
+  y <- nlist(formula, family, mv, rescor, mecor)
+  y$cov_ranef <- x$cov_ranef
   class(y) <- "brmsterms"
   
   if (check_response) {
     # extract response variables
     y$respform <- validate_resp_formula(formula, empty_ok = FALSE)
     if (mv) {
-      y$resp <- parse_resp(y$respform) 
+      y$resp <- terms_resp(y$respform) 
     } else {
       y$resp <- ""
     }
   }
   
   # extract addition arguments
-  adforms <- parse_ad(formula, family, check_response)
+  adforms <- terms_ad(formula, family, check_response)
   advars <- str2formula(ulapply(adforms, all_vars))
   y$adforms[names(adforms)] <- adforms
   
@@ -109,9 +117,9 @@ parse_bf.brmsformula <- function(formula, family = NULL, autocor = NULL,
   y$dpars <- named_list(dpars)
   for (dp in dpars) {
     if (get_nl(dpar_forms[[dp]])) {
-      y$dpars[[dp]] <- parse_nlf(dpar_forms[[dp]], nlpars, resp)
+      y$dpars[[dp]] <- terms_nlf(dpar_forms[[dp]], nlpars, resp)
     } else {
-      y$dpars[[dp]] <- parse_lf(dpar_forms[[dp]])
+      y$dpars[[dp]] <- terms_lf(dpar_forms[[dp]])
     }
     y$dpars[[dp]]$family <- dpar_family(family, dp)
     y$dpars[[dp]]$dpar <- dp
@@ -120,6 +128,7 @@ parse_bf.brmsformula <- function(formula, family = NULL, autocor = NULL,
       y$dpars[[dp]]$respform <- y$respform
       y$dpars[[dp]]$adforms <- y$adforms
     }
+    check_cs(y$dpars[[dp]])
   }
   
   y$nlpars <- named_list(nlpars)
@@ -132,12 +141,13 @@ parse_bf.brmsformula <- function(formula, family = NULL, autocor = NULL,
         attr(nlpar_forms[[nlp]], "center") <- FALSE
       }
       if (get_nl(nlpar_forms[[nlp]])) {
-        y$nlpars[[nlp]] <- parse_nlf(nlpar_forms[[nlp]], nlpars, resp)
+        y$nlpars[[nlp]] <- terms_nlf(nlpar_forms[[nlp]], nlpars, resp)
       } else {
-        y$nlpars[[nlp]] <- parse_lf(nlpar_forms[[nlp]])
+        y$nlpars[[nlp]] <- terms_lf(nlpar_forms[[nlp]])
       }
       y$nlpars[[nlp]]$nlpar <- nlp
       y$nlpars[[nlp]]$resp <- resp
+      check_cs(y$nlpars[[nlp]])
     }
     used_nlpars <- ulapply(c(y$dpars, y$nlpars), "[[", "used_nlpars")
     unused_nlpars <- setdiff(nlpars, used_nlpars)
@@ -184,29 +194,30 @@ parse_bf.brmsformula <- function(formula, family = NULL, autocor = NULL,
     y$fdpars[[dp]] <- list(value = x$pfix[[dp]], dpar = dp)
   }
   check_fdpars(y$fdpars)
-  # check for illegal use of cs terms
-  if (has_cs(y) && !(is.null(family) || allow_cs(family))) {
-    stop2("Category specific effects require families ", 
-          "'sratio', 'cratio', or 'acat'.")
-  }
   
   # make a formula containing all required variables
+  unused_vars <- all_vars(attr(x$formula, "unused"))
   lhsvars <- if (resp_rhs_all) all_vars(y$respform)
   y$allvars <- allvars_formula(
     lhsvars, advars, lapply(y$dpars, get_allvars), 
-    lapply(y$nlpars, get_allvars), y$time$allvars
+    lapply(y$nlpars, get_allvars), y$time$allvars,
+    unused_vars
   )
   if (check_response) {
-    y$allvars <- update(y$respform, y$allvars) 
+    # add y$respform to the left-hand side of y$allvars
+    # avoid using update.formula as it is inefficient for longer formulas
+    formula_allvars <- y$respform 
+    formula_allvars[[3]] <- y$allvars[[2]]
+    y$allvars <- formula_allvars
   }
   environment(y$allvars) <- environment(formula)
   y
 }
 
-#' @rdname parse_bf
+#' @rdname brmsterms
 #' @export
-parse_bf.mvbrmsformula <- function(formula, family = NULL, autocor = NULL, ...) {
-  x <- validate_formula(formula, family = family, autocor = autocor)
+brmsterms.mvbrmsformula <- function(formula, ...) {
+  x <- validate_formula(formula)
   x$rescor <- isTRUE(x$rescor)
   x$mecor <- isTRUE(x$mecor)
   out <- structure(list(), class = "mvbrmsterms")
@@ -214,28 +225,32 @@ parse_bf.mvbrmsformula <- function(formula, family = NULL, autocor = NULL, ...)
   for (i in seq_along(out$terms)) {
     x$forms[[i]]$rescor <- x$rescor
     x$forms[[i]]$mecor <- x$mecor
-    out$terms[[i]] <- parse_bf(x$forms[[i]], mv = TRUE, ...)
+    x$forms[[i]]$mv <- TRUE
+    out$terms[[i]] <- brmsterms(x$forms[[i]], ...)
   }
   out$allvars <- allvars_formula(lapply(out$terms, get_allvars))
   # required to find variables used solely in the response part
   lhs_resp <- function(x) deparse_combine(lhs(x$respform)[[2]])
   out$respform <- paste0(ulapply(out$terms, lhs_resp), collapse = ",")
-  out$respform <- formula(paste0("cbind(", out$respform, ") ~ 1"))
+  out$respform <- formula(paste0("mvbind(", out$respform, ") ~ 1"))
   out$responses <- ulapply(out$terms, "[[", "resp")
   out$rescor <- x$rescor
   out$mecor <- x$mecor
+  out$cov_ranef <- x$cov_ranef
   out
 }
 
 # parse linear/additive formulas
 # @param formula an ordinary model formula
 # @return a 'btl' object
-parse_lf <- function(formula) {
+terms_lf <- function(formula) {
   formula <- rhs(as.formula(formula))
   y <- nlist(formula)
+  formula <- terms(formula)
+  check_accidental_helper_functions(formula)
   types <- setdiff(all_term_types(), excluded_term_types(formula))
   for (t in types) {
-    tmp <- do_call(paste0("parse_", t), list(formula))
+    tmp <- do_call(paste0("terms_", t), list(formula))
     if (is.data.frame(tmp) || is.formula(tmp)) {
       y[[t]] <- tmp 
     }
@@ -255,7 +270,7 @@ parse_lf <- function(formula) {
 # @param nlpars names of all non-linear parameters
 # @param resp optional name of a response variable
 # @return a 'btnl' object
-parse_nlf <- function(formula, nlpars, resp = "") {
+terms_nlf <- function(formula, nlpars, resp = "") {
   if (!length(nlpars)) {
     stop2("No non-linear parameters specified.")
   }
@@ -267,7 +282,7 @@ parse_nlf <- function(formula, nlpars, resp = "") {
   covars <- setdiff(all_vars, nlpars)
   y$covars <- structure(str2formula(covars), int = FALSE)
   if (!"ac" %in% excluded_term_types(formula)) {
-    y$ac <- parse_ac(attr(formula, "autocor")) 
+    y$ac <- terms_ac(attr(formula, "autocor")) 
   }
   y$allvars <- allvars_formula(covars, get_allvars(y$ac))
   environment(y$allvars) <- environment(formula)
@@ -277,7 +292,7 @@ parse_nlf <- function(formula, nlpars, resp = "") {
 
 # extract addition arguments out of formula
 # @return a list of formulas each containg a single addition term
-parse_ad <- function(formula, family = NULL, check_response = TRUE) {
+terms_ad <- function(formula, family = NULL, check_response = TRUE) {
   x <- list()
   ad_funs <- lsp("brms", what = "exports", pattern = "^resp_")
   ad_funs <- sub("^resp_", "", ad_funs)
@@ -287,7 +302,11 @@ parse_ad <- function(formula, family = NULL, check_response = TRUE) {
     ad <- get_matches("(?<=\\|)[^~]*(?=~)", str_formula, perl = TRUE)
     valid_ads <- family_info(family, "ad")
     if (length(ad)) {
-      ad_terms <- attr(terms(formula(paste("~", ad))), "term.labels")
+      ad_terms <- terms(str2formula(ad))
+      if (length(attr(ad_terms, "offset"))) {
+        stop2("Offsets are not allowed in addition terms.")
+      }
+      ad_terms <- attr(ad_terms, "term.labels")
       for (a in ad_funs) {
         matches <- grep(paste0("^(resp_)?", a, "\\(.*\\)$"), ad_terms)
         if (length(matches) == 1L) {
@@ -323,17 +342,20 @@ parse_ad <- function(formula, family = NULL, check_response = TRUE) {
 }
 
 # extract fixed effects terms
-parse_fe <- function(formula) {
+terms_fe <- function(formula) {
+  if (!is.terms(formula)) {
+    formula <- terms(formula)
+  }
   all_terms <- all_terms(formula)
   sp_terms <- find_terms(all_terms, "all", complete = FALSE)
   re_terms <- all_terms[grepl("\\|", all_terms)]
-  int_term <- attr(terms(formula), "intercept")
+  int_term <- attr(formula, "intercept")
   fe_terms <- setdiff(all_terms, c(sp_terms, re_terms))
   out <- paste(c(int_term, fe_terms), collapse = "+")
   out <- str2formula(out)
   attr(out, "allvars") <- allvars_formula(out)
   attr(out, "decomp") <- get_decomp(formula)
-  if (has_rsv_intercept(out)) {
+  if (has_rsv_intercept(out, has_intercept(formula))) {
     attr(out, "int") <- FALSE
   }
   if (no_cmc(formula)) {
@@ -350,7 +372,7 @@ parse_fe <- function(formula) {
 
 # gather information of group-level terms
 # @return a data.frame with one row per group-level term
-parse_re <- function(formula) {
+terms_re <- function(formula) {
   re_terms <- get_re_terms(formula, brackets = FALSE)
   if (!length(re_terms)) {
     return(NULL)
@@ -360,15 +382,12 @@ parse_re <- function(formula) {
   out <- allvars <- vector("list", length(re_terms))
   type <- attr(re_terms, "type")
   for (i in seq_along(re_terms)) {
-    id <- gsub("\\|", "", re_parts$mid[i])
-    if (!nzchar(id)) id <- NA
     gcall <- eval2(re_parts$rhs[i])
     form <- str2formula(re_parts$lhs[i])
     group <- paste0(gcall$type, collapse(gcall$groups))
     out[[i]] <- data.frame(
-      group = group, gtype = gcall$type, 
-      gn = i, id = id, type = type[i],
-      cor = substr(re_parts$mid[i], 1, 2) != "||",
+      group = group, gtype = gcall$type, gn = i, 
+      id = gcall$id, type = type[i], cor = gcall$cor,
       stringsAsFactors = FALSE
     )
     out[[i]]$gcall <- list(gcall)
@@ -393,12 +412,12 @@ parse_re <- function(formula) {
 }
 
 # extract category specific terms for ordinal models
-parse_cs <- function(formula) {
+terms_cs <- function(formula) {
   out <- find_terms(formula, "cs")
   if (!length(out)) {
     return(NULL)
   }
-  out <- ulapply(out, eval2)
+  out <- ulapply(out, eval2, envir = environment())
   out <- str2formula(out)
   attr(out, "allvars") <- allvars_formula(out)
   # do not test whether variables were supplied to 'cs'
@@ -408,15 +427,15 @@ parse_cs <- function(formula) {
 }
 
 # extract special effects terms 
-parse_sp <- function(formula) {
+terms_sp <- function(formula) {
   types <- c("mo", "me", "mi")
   out <- find_terms(formula, types, complete = FALSE)
   if (!length(out)) {
     return(NULL) 
   }
-  uni_mo <- rm_wsp(get_matches_expr(regex_sp("mo"), out))
-  uni_me <- rm_wsp(get_matches_expr(regex_sp("me"), out))
-  uni_mi <- rm_wsp(get_matches_expr(regex_sp("mi"), out))
+  uni_mo <- trim_wsp(get_matches_expr(regex_sp("mo"), out))
+  uni_me <- trim_wsp(get_matches_expr(regex_sp("me"), out))
+  uni_mi <- trim_wsp(get_matches_expr(regex_sp("mi"), out))
   # remove the intercept as it is handled separately
   out <- str2formula(c("0", out))
   attr(out, "int") <- FALSE
@@ -430,7 +449,7 @@ parse_sp <- function(formula) {
 }
 
 # extract spline terms
-parse_sm <- function(formula) {
+terms_sm <- function(formula) {
   out <- find_terms(formula, "sm")
   if (!length(out)) {
     return(NULL)
@@ -445,12 +464,12 @@ parse_sm <- function(formula) {
 }
 
 # extract gaussian process terms
-parse_gp <- function(formula) {
+terms_gp <- function(formula) {
   out <- find_terms(formula, "gp")
   if (!length(out)) {
     return(NULL)
   }
-  eterms <- lapply(out, eval2)
+  eterms <- lapply(out, eval2, envir = environment())
   covars <- lapply(eterms, "[[", "term")
   byvars <- lapply(eterms, "[[", "by")
   allvars <- str2formula(unlist(c(covars, byvars)))
@@ -464,13 +483,13 @@ parse_gp <- function(formula) {
 }
 
 # extract autocorrelation terms
-parse_ac <- function(formula) {
+terms_ac <- function(formula) {
   autocor <- attr(formula, "autocor")
   out <- c(find_terms(formula, "ac"), find_terms(autocor, "ac"))
   if (!length(out)) {
     return(NULL)
   }
-  eterms <- lapply(out, eval2)
+  eterms <- lapply(out, eval2, envir = environment())
   allvars <- unlist(c(
     lapply(eterms, "[[", "time"),
     lapply(eterms, "[[", "gr")
@@ -482,13 +501,15 @@ parse_ac <- function(formula) {
 }
 
 # extract offset terms
-parse_offset <- function(formula) {
-  terms <- terms(as.formula(formula))
-  pos <- attr(terms, "offset")
+terms_offset <- function(formula) {
+  if (!is.terms(formula)) {
+    formula <- terms(as.formula(formula)) 
+  }
+  pos <- attr(formula, "offset")
   if (is.null(pos)) {
     return(NULL)
   }
-  vars <- attr(terms, "variables")
+  vars <- attr(formula, "variables")
   out <- ulapply(pos, function(i) deparse(vars[[i + 1]]))
   out <- str2formula(out)
   attr(out, "allvars") <- str2formula(all_vars(out))
@@ -496,7 +517,7 @@ parse_offset <- function(formula) {
 }
 
 # extract multiple covariates in multi-membership terms
-parse_mmc <- function(formula) {
+terms_mmc <- function(formula) {
   out <- find_terms(formula, "mmc")
   if (!length(out)) {
     return(NULL)
@@ -510,7 +531,7 @@ parse_mmc <- function(formula) {
 
 # extract response variable names
 # assumes multiple response variables to be combined via mvbind
-parse_resp <- function(formula, check_names = TRUE) {
+terms_resp <- function(formula, check_names = TRUE) {
   formula <- lhs(as.formula(formula))
   if (is.null(formula)) {
     return(NULL)
@@ -520,14 +541,8 @@ parse_resp <- function(formula, check_names = TRUE) {
     out <- deparse_no_string(expr)
   } else {
     str_fun <- deparse_no_string(expr[[1]]) 
-    use_mvbind <- identical(str_fun, "mvbind")
-    use_cbind <- identical(str_fun, "cbind")
-    if (use_mvbind) {
-      out <- ulapply(expr[-1], deparse_no_string)
-    } else if (use_cbind) {
-      # deprecated as of brms 2.7.2
-      warning2("Using 'cbind' for multivariate models is ", 
-               "deprecated. Please use 'mvbind' instead.")
+    used_mvbind <- grepl("^(brms:::?)?mvbind$", str_fun)
+    if (used_mvbind) {
       out <- ulapply(expr[-1], deparse_no_string)
     } else {
       out <- deparse_no_string(expr) 
@@ -543,7 +558,7 @@ parse_resp <- function(formula, check_names = TRUE) {
 #' 
 #' @param x An \R object
 #' 
-#' @seealso \code{\link[brms:parse_bf]{parse_bf}}
+#' @seealso \code{\link[brms:brmsterms]{brmsterms}}
 #' 
 #' @export
 is.brmsterms <- function(x) {
@@ -554,7 +569,7 @@ is.brmsterms <- function(x) {
 #' 
 #' @param x An \R object
 #' 
-#' @seealso \code{\link[brms:parse_bf]{parse_bf}}
+#' @seealso \code{\link[brms:brmsterms]{brmsterms}}
 #' 
 #' @export
 is.mvbrmsterms <- function(x) {
@@ -698,8 +713,7 @@ allvars_formula <- function(...) {
     stop2("The following variable names are invalid: ",
           collapse_comma(invalid_vars)) 
   }
-  out <- str2formula(c(out, all_vars))
-  update(out, ~ .)
+  str2formula(c(out, all_vars))
 }
 
 # conveniently extract a formula of all relevant variables
@@ -715,8 +729,8 @@ get_allvars <- function(x, type = "") {
   if (is.null(out) && is.formula(x)) {
     type <- as_one_character(type)
     type <- str_if(nzchar(type), type, "fe")
-    parse_fun <- get(paste0("parse_", type), mode = "function")
-    out <- attr(parse_fun(x), "allvars")
+    terms_fun <- get(paste0("terms_", type), mode = "function")
+    out <- attr(terms_fun(x), "allvars")
   }
   stopifnot(is.null(out) || is.formula(out))
   out
@@ -735,6 +749,20 @@ plus_rhs <- function(x) {
   out
 }
 
+# like stats::terms but keeps attributes if possible
+terms <- function(formula, ...) {
+  old_attributes <- attributes(formula)
+  formula <- stats::terms(formula, ...)
+  new_attributes <- attributes(formula)
+  sel_names <- setdiff(names(old_attributes), names(new_attributes))
+  attributes(formula)[sel_names] <- old_attributes[sel_names]
+  formula
+}
+
+is.terms <- function(x) {
+  inherits(x, "terms")
+}
+
 # combine formulas for distributional parameters
 # @param formula1 primary formula from which to take the RHS
 # @param formula2 secondary formula used to update the RHS of formula1
@@ -742,7 +770,7 @@ plus_rhs <- function(x) {
 # @param update a flag to indicate whether updating should be allowed.
 #   Defaults to FALSE to maintain backwards compatibility
 # @return a formula object 
-combine_formulas <- function(formula1, formula2, lhs, update = FALSE) {
+combine_formulas <- function(formula1, formula2, lhs = "", update = FALSE) {
   stopifnot(is.formula(formula1))
   stopifnot(is.null(formula2) || is.formula(formula2))
   lhs <- as_one_character(lhs)
@@ -779,6 +807,15 @@ has_terms <- function(formula) {
     length(attr(terms, "offset")) 
 }
 
+# has a linear formula any terms except overall effects?
+has_special_terms <- function(x) {
+  if (!is.btl(x)) {
+    return(FALSE)
+  }
+  special_terms <- c("sp", "sm", "gp", "ac", "cs", "offset")
+  NROW(x[["re"]]) > 0 || any(lengths(x[special_terms]))
+}
+
 # indicate if the predictor term belongs to a non-linear parameter
 is_nlpar <- function(x) {
   isTRUE(nzchar(x[["nlpar"]]))
@@ -830,12 +867,12 @@ get_effect.brmsfit <- function(x, ...) {
 
 #' @export
 get_effect.brmsformula <- function(x, ...) {
-  get_effect(parse_bf(x), ...)
+  get_effect(brmsterms(x), ...)
 }
 
 #' @export
 get_effect.mvbrmsformula <- function(x, ...) {
-  get_effect(parse_bf(x), ...)
+  get_effect(brmsterms(x), ...)
 }
 
 #' @export
@@ -848,17 +885,13 @@ get_effect.mvbrmsterms <- function(x, ...) {
 # @param all logical; include effects of nlpars and dpars?
 # @return a list of formulas
 #' @export
-get_effect.brmsterms <- function(x, target = "fe", all = TRUE, ...) {
-  if (all) {
-    out <- named_list(c(names(x$dpars), names(x$nlpars)))
-    for (dp in names(x$dpars)) {
-      out[[dp]] <- get_effect(x$dpars[[dp]], target = target)
-    }
-    for (nlp in names(x$nlpars)) {
-      out[[nlp]] <- get_effect(x$nlpars[[nlp]], target = target)
-    }
-  } else {
-    out <- get_effect(x$dpars[["mu"]], target = target)
+get_effect.brmsterms <- function(x, target = "fe", ...) {
+  out <- named_list(c(names(x$dpars), names(x$nlpars)))
+  for (dp in names(x$dpars)) {
+    out[[dp]] <- get_effect(x$dpars[[dp]], target = target)
+  }
+  for (nlp in names(x$nlpars)) {
+    out[[nlp]] <- get_effect(x$nlpars[[nlp]], target = target)
   }
   unlist(out, recursive = FALSE)
 }
@@ -877,10 +910,10 @@ all_terms <- function(x) {
   if (!length(x)) {
     return(character(0))
   }
-  if (!inherits(x, "terms")) {
+  if (!is.terms(x)) {
     x <- terms(as.formula(x))
   }
-  rm_wsp(attr(x, "term.labels"))
+  trim_wsp(attr(x, "term.labels"))
 }
 
 # generate a regular expression to extract special terms
@@ -904,7 +937,7 @@ regex_sp <- function(type = "all") {
     type <- names(funs)
   }
   funs <- funs[type]
-  allow_colon <- c("cs", "mmc")
+  allow_colon <- c("cs", "mmc", "ac")
   inner <- ifelse(names(funs) %in% allow_colon, ".*", "[^:]*")
   out <- paste0("^(", funs, ")\\(", inner, "\\)$")
   paste0("(", out, ")", collapse = "|")
@@ -936,7 +969,7 @@ find_terms <- function(x, type, complete = TRUE, ranef = FALSE) {
     inv <- out[lengths(matches) > 1L]
     if (!length(inv)) {
       # each term must be exactly equal to the special function call
-      inv <- out[rm_wsp(unlist(matches)) != out]
+      inv <- out[trim_wsp(unlist(matches)) != out]
     }
     if (length(inv)) {
       stop2("The term '", inv[1], "' is invalid in brms syntax.")
@@ -953,10 +986,10 @@ find_terms <- function(x, type, complete = TRUE, ranef = FALSE) {
 validate_terms <- function(x) {
   no_int <- no_int(x)
   no_cmc <- no_cmc(x)
-  if (is.formula(x) && !inherits(x, "terms")) {
+  if (is.formula(x) && !is.terms(x)) {
     x <- terms(x)
   }
-  if (!inherits(x, "terms")) {
+  if (!is.terms(x)) {
     return(NULL)
   }
   if (no_int || !has_intercept(x) && no_cmc) {
@@ -969,32 +1002,48 @@ validate_terms <- function(x) {
  
 # checks if the formula contains an intercept
 has_intercept <- function(formula) {
-  formula <- as.formula(formula)
-  try_terms <- try(terms(formula), silent = TRUE)
-  if (is(try_terms, "try-error")) {
-    out <- FALSE
+  if (is.terms(formula)) {
+    out <- as.logical(attr(formula, "intercept"))
   } else {
-    out <- as.logical(attr(try_terms, "intercept"))
+    formula <- as.formula(formula)
+    try_terms <- try(terms(formula), silent = TRUE)
+    if (is(try_terms, "try-error")) {
+      out <- FALSE
+    } else {
+      out <- as.logical(attr(try_terms, "intercept"))
+    } 
   }
   out
 }
 
 # check if model makes use of the reserved intercept variables
-has_rsv_intercept <- function(formula) {
+# @param has_intercept does the model have an intercept?
+#   if NULL this will be inferred from formula itself
+has_rsv_intercept <- function(formula, has_intercept = NULL) {
+  .has_rsv_intercept <- function(terms, has_intercept) {
+    has_intercept <- as_one_logical(has_intercept)
+    intercepts <- c("intercept", "Intercept")
+    out <- !has_intercept && any(intercepts %in% all_vars(rhs(terms)))
+    return(out)
+  }
+  if (is.terms(formula)) {
+    if (is.null(has_intercept)) {
+      has_intercept <- has_intercept(formula)
+    }
+    return(.has_rsv_intercept(formula, has_intercept))
+  }
   formula <- try(as.formula(formula), silent = TRUE)
   if (is(formula, "try-error")) {
-    out <- FALSE
-  } else {
+    return(FALSE)
+  } 
+  if (is.null(has_intercept)) {
     try_terms <- try(terms(formula), silent = TRUE)
     if (is(try_terms, "try-error")) {
-      out <- FALSE
-    } else {
-      has_intercept <- attr(try_terms, "intercept")
-      intercepts <- c("intercept", "Intercept")
-      out <- !has_intercept && any(intercepts %in% all_vars(rhs(formula)))
-    }
+      return(FALSE)
+    } 
+    has_intercept <- has_intercept(try_terms)
   }
-  out
+  .has_rsv_intercept(formula, has_intercept)
 }
 
 # names of reserved variables
@@ -1012,12 +1061,53 @@ rsv_vars <- function(bterms) {
   out
 }
 
-# check if category specific effects are present in the model
+# are category specific effects present?
 has_cs <- function(bterms) {
   length(get_effect(bterms, target = "cs")) > 0L ||
     any(get_re(bterms)$type %in% "cs")
 }
 
+# check if category specific effects are allowed
+check_cs <- function(bterms) {
+  stopifnot(is.btl(bterms) || is.btnl(bterms))
+  if (has_cs(bterms)) {
+    if (!is_equal(dpar_class(bterms$dpar), "mu")) {
+      stop2("Category specific effects are only supported ", 
+            "for the main parameter 'mu'.")
+    }
+    if (!(is.null(bterms$family) || allow_cs(bterms$family))) {
+      stop2("Category specific effects are not supported for this family.")
+    }
+    if (needs_ordered_cs(bterms$family)) {
+      warning2("Category specific effects for this family should be ",
+               "considered experimental and may have convergence issues.")
+    }
+  }
+  invisible(NULL)
+}
+
+# check for the presence of helper functions accidentally used
+# within a formula instead of added to bf(). See #1103
+check_accidental_helper_functions <- function(formula) {
+  terms <- all_terms(formula)
+  # see help("brmsformula-helpers") for the list of functions
+  funs <- c("nlf", "lf", "acformula", "set_nl", "set_rescor", "set_mecor")
+  regex <- paste0("(", funs, ")", collapse = "|")
+  regex <- paste0("^(", regex, ")\\(")
+  matches <- get_matches(regex, terms, first = TRUE)
+  matches <- sub("\\($", "", matches)
+  matches <- unique(matches)
+  matches <- matches[nzchar(matches)]
+  for (m in matches) {
+    loc <- utils::find(m, mode = "function")
+    if (is_equal(loc[1], "package:brms")) {
+      stop2("Function '", m, "' should not be part of the right-hand side ",
+            "of a formula. See help('brmsformula-helpers') for the correct syntax.")
+    }
+  }
+  invisible(TRUE)
+}
+
 # extract elements from objects
 # @param x object from which to extract elements
 # @param name name of the element to be extracted
