@@ -3,24 +3,21 @@
 # @param bterms object of class brmsterms
 # @param na.action function defining how to treat NAs
 # @param drop.unused.levels should unused factor levels be removed?
-# @param terms_attr a list of attributes of the terms object of 
+# @param attr_terms a list of attributes of the terms object of 
 #   the original model.frame; only used with newdata;
 #   this ensures that (1) calls to 'poly' work correctly
 #   and (2) that the number of variables matches the number 
 #   of variable names; fixes issue #73
 # @param knots: a list of knot values for GAMMs
 # @return model.frame for use in brms functions
-validate_data <- function(data, bterms, na.action = na.omit2,
-                          drop.unused.levels = TRUE,
-                          terms_attr = NULL, knots = NULL) {
+validate_data <- function(data, bterms, data2 = list(), knots = NULL,
+                          na.action = na.omit2, drop.unused.levels = TRUE, 
+                          attr_terms = NULL) {
   if (missing(data)) {
     stop2("Data must be specified using the 'data' argument.")
   }
-  if (isTRUE(attr(data, "brmsframe"))) {
-    return(data)
-  }
   if (is.null(knots)) {
-    knots <- attr(data, "knots", TRUE)
+    knots <- get_knots(data)
   }
   data <- try(as.data.frame(data), silent = TRUE)
   if (is(data, "try-error")) {
@@ -29,45 +26,59 @@ validate_data <- function(data, bterms, na.action = na.omit2,
   if (!isTRUE(nrow(data) > 0L)) {
     stop2("Argument 'data' does not contain observations.")
   }
-  bterms$allvars <- terms(bterms$allvars)
-  attributes(bterms$allvars)[names(terms_attr)] <- terms_attr
   data <- data_rsv_intercept(data, bterms = bterms)
-  missing_vars <- setdiff(all.vars(bterms$allvars), names(data))
+  all_vars_formula <- bterms$allvars
+  missing_vars <- setdiff(all_vars(all_vars_formula), names(data))
   if (length(missing_vars)) {
-    stop2("The following variables are missing in 'data':\n",
-          collapse_comma(missing_vars))
-  }
-  for (v in intersect(vars_keep_na(bterms), names(data))) {
-    attr(data[[v]], "keep_na") <- TRUE
+    missing_vars2 <- setdiff(missing_vars, names(data2))
+    if (length(missing_vars2)) {
+      stop2("The following variables can neither be found in ",
+            "'data' nor in 'data2':\n", collapse_comma(missing_vars2)) 
+    }
+    # all initially missing variables can be found in 'data2'
+    # they are not necessarily of the length required for 'data'
+    # so need to be excluded from the evaluation of 'model.frame'
+    missing_vars_formula <- paste0(". ~ . ", collapse(" - ", missing_vars))
+    all_vars_formula <- update(all_vars_formula, missing_vars_formula)
   }
+  all_vars_terms <- terms(all_vars_formula)
+  # ensure that 'data2' comes first in the search path 
+  # during the evaluation of model.frame
+  terms_env <- environment(all_vars_terms)
+  environment(all_vars_terms) <- as.environment(as.list(data2))
+  parent.env(environment(all_vars_terms)) <- terms_env
+  attributes(all_vars_terms)[names(attr_terms)] <- attr_terms
+  # 'terms' prevents correct validation in 'model.frame'
+  attr(data, "terms") <- NULL
   data <- model.frame(
-    bterms$allvars, data, na.action = na.action,
+    all_vars_terms, data, na.action = na.pass,
     drop.unused.levels = drop.unused.levels
   )
+  data <- na.action(data, bterms = bterms)
   if (any(grepl("__|_$", colnames(data)))) {
     stop2("Variable names may not contain double underscores ",
           "or underscores at the end.")
   }
+  if (!isTRUE(nrow(data) > 0L)) {
+    stop2("All observations in the data were removed ", 
+          "presumably because of NA values.") 
+  }
   groups <- get_group_vars(bterms)
   data <- combine_groups(data, groups)
   data <- fix_factor_contrasts(data, ignore = groups)
   attr(data, "knots") <- knots
-  attr(data, "brmsframe") <- TRUE
   data
 }
 
 # validate the 'data2' argument
 # @param data2 a named list of data objects
-# @param bterms object returned by 'parse_bf'
+# @param bterms object returned by 'brmsterms'
 # @param ... more named list to pass objects to data2 from other sources
 #   only required for backwards compatibility with deprecated arguments
 # @return a validated named list of data objects
 validate_data2 <- function(data2, bterms, ...) {
   # TODO: specify spline-related matrices in 'data2'
-  # TODO: specify 'knots' in 'data2'?
-  if (isTRUE(attr(data2, "valid"))) {
-    return(data2)
-  }
+  # this requires adding another parser layer with bterms and data as input
   if (is.null(data2)) {
     data2 <- list()
   }
@@ -102,7 +113,13 @@ validate_data2 <- function(data2, bterms, ...) {
     data2[[M]] <- validate_fcor_matrix(get_from_data2(M, data2))
     attr(data2[[M]], "obs_based_matrix") <- TRUE
   }
-  structure(data2, valid = TRUE)
+  # validate within-group covariance matrices
+  cov_names <- ulapply(get_re(bterms)$gcall, "[[", "cov")
+  cov_names <- cov_names[nzchar(cov_names)]
+  for (cov in cov_names) {
+    data2[[cov]] <- validate_recov_matrix(get_from_data2(cov, data2))
+  }
+  data2
 }
 
 # get an object from the 'data2' argument
@@ -162,11 +179,11 @@ combine_groups <- function(data, ...) {
   for (i in seq_along(group)) {
     sgroup <- unlist(strsplit(group[[i]], ":"))
     if (length(sgroup) > 1L && !group[[i]] %in% names(data)) {
-      new.var <- get(sgroup[1], data)
+      new_var <- get(sgroup[1], data)
       for (j in 2:length(sgroup)) {
-        new.var <- paste0(new.var, "_", get(sgroup[j], data))
+        new_var <- paste0(new_var, "_", get(sgroup[j], data))
       }
-      data[[group[[i]]]] <- new.var
+      data[[group[[i]]]] <- new_var
     }
   } 
   data
@@ -174,19 +191,19 @@ combine_groups <- function(data, ...) {
 
 # hard code factor contrasts to be independent of the global "contrasts" option
 # @param data data.frame to be updated
-# @param optdata: optional data.frame from which contrasts are taken if present
+# @param olddata: optional data.frame from which contrasts are taken if present
 # @param ignore: names of variables for which not to fix contrasts
 # @return 'data' with amended contrasts attributes
-fix_factor_contrasts <- function(data, optdata = NULL, ignore = NULL) {
+fix_factor_contrasts <- function(data, olddata = NULL, ignore = NULL) {
   stopifnot(is(data, "data.frame"))
-  stopifnot(is.null(optdata) || is.list(optdata))
-  optdata <- as.data.frame(optdata)  # fixes issue #105
+  stopifnot(is.null(olddata) || is.list(olddata))
+  olddata <- as.data.frame(olddata)  # fixes issue #105
   for (i in seq_along(data)) {
     needs_contrast <- is.factor(data[[i]]) && !names(data)[i] %in% ignore
     if (needs_contrast && is.null(attr(data[[i]], "contrasts"))) {
-      old_contrasts <- attr(optdata[[names(data)[i]]], "contrasts")
+      old_contrasts <- attr(olddata[[names(data)[i]]], "contrasts")
       if (!is.null(old_contrasts)) {
-        # take contrasts from optdata
+        # take contrasts from olddata
         contrasts(data[[i]]) <- old_contrasts
       } else if (length(unique(data[[i]])) > 1L) {
         # avoid error when supplying only a single level
@@ -234,23 +251,24 @@ order_data <- function(data, bterms) {
 
 # subset data according to addition argument 'subset'
 subset_data <- function(data, bterms) {
-  if (is.formula(bterms$adforms$subset)) {
+  if (has_subset(bterms)) {
     # only evaluate a subset of the data
-    subset <- eval_rhs(bterms$adforms$subset)
-    subset <- as.logical(eval2(subset$vars$subset, data))
+    subset <- as.logical(get_ad_values(bterms, "subset", "subset", data))
     if (length(subset) != nrow(data)) {
       stop2("Length of 'subset' does not match the rows of 'data'.")
     }
     if (anyNA(subset)) {
       stop2("Subset variables may not contain NAs.")
     }
+    # cross-formula indexing is no longer trivial for subsetted models
+    check_cross_formula_indexing(bterms)
     data <- data[subset, , drop = FALSE]
   }
   if (!NROW(data)) {
     stop2(
       "All rows of 'data' were removed via 'subset'. ",
       "Please make sure that variables do not contain NAs ",
-      "even in rows unused by the subsetted model. ",
+      "for observations in which they are supposed to be used. ",
       "Please also make sure that each subset variable is ",
       "TRUE for at least one observation."
     )
@@ -258,6 +276,64 @@ subset_data <- function(data, bterms) {
   data
 }
 
+# like stats:::na.omit.data.frame but allows to certain NA values
+na.omit2 <- function(object, bterms, ...) {
+  stopifnot(is.data.frame(object))
+  nobs <- nrow(object)
+  if (is.mvbrmsterms(bterms)) {
+    responses <- names(bterms$terms)
+    subsets <- lapply(bterms$terms, get_ad_values, "subset", "subset", object)
+    vars_sub <- lapply(bterms$terms, function(x) all_vars(x$allvars)) 
+  }
+  vars_keep_na <- vars_keep_na(bterms)
+  omit <- logical(nobs)
+  for (v in names(object)) {
+    x <- object[[v]]
+    vars_v <- all_vars(v)
+    keep_all_na <- all(vars_v %in% vars_keep_na)
+    if (!is.atomic(x) || keep_all_na) {
+      next
+    }
+    if (!is.mvbrmsterms(bterms)) {
+      # remove all NAs in this variable
+      keep_na <- rep(FALSE, nobs)
+    } else {
+      # allow to retain NAs in subsetted variables
+      keep_na <- rep(TRUE, nobs)
+      for (r in responses) {
+        if (any(vars_v %in% vars_sub[[r]])) {
+          if (!is.null(subsets[[r]])) {
+            # keep NAs ignored because of 'subset'
+            keep_na <- keep_na & !subsets[[r]]
+          } else {
+            # remove all NAs in this variable
+            keep_na <- keep_na & FALSE
+          }
+        }
+      }
+    }
+    is_na <- is.na(x)
+    d <- dim(is_na)
+    if (is.null(d) || length(d) != 2L) {
+      omit <- omit | (is_na & !keep_na)
+    } else {
+      for (ii in seq_len(d[2L])) {
+        omit <- omit | (is_na[, ii] & !keep_na)
+      } 
+    }
+  }
+  if (any(omit > 0L)) {
+    out <- object[!omit, , drop = FALSE]
+    temp <- setNames(seq(omit)[omit], attr(object, "row.names")[omit])
+    attr(temp, "class") <- "omit"
+    attr(out, "na.action") <- temp
+    warning2("Rows containing NAs were excluded from the model.")
+  } else {
+    out <- object
+  }
+  out
+}
+
 # get a single value per group 
 # @param x vector of values to extract one value per group
 # @param gr vector of grouping values
@@ -273,19 +349,36 @@ get_one_value_per_group <- function(x, gr) {
   out
 }
 
+# extract knots values for use in spline terms
+get_knots <- function(data) {
+  attr(data, "knots", TRUE)
+}
+
+# extract name of the data as originally passed by the user
+get_data_name <- function(data) {
+  out <- attr(data, "data_name", TRUE)
+  if (is.null(out)) {
+    out <- "NULL"
+  }
+  out
+}
+
 #' Validate New Data
 #' 
 #' Validate new data passed to post-processing methods of \pkg{brms}. Unless you
 #' are a package developer, you will rarely need to call \code{validate_newdata}
 #' directly.
 #' 
-#' @inheritParams extract_draws
+#' @inheritParams prepare_predictions
 #' @param newdata A \code{data.frame} containing new data to be validated.
 #' @param object A \code{brmsfit} object.
 #' @param check_response Logical; Indicates if response variables should
 #'   be checked as well. Defaults to \code{TRUE}.
-#' @param all_group_vars Optional names of grouping variables to be validated.
+#' @param group_vars Optional names of grouping variables to be validated.
 #'   Defaults to all grouping variables in the model.
+#' @param req_vars Optional names of variables required in \code{newdata}.
+#'   If \code{NULL} (the default), all variables in the original data
+#'   are required (unless ignored for some other reason).
 #' @param ... Currently ignored.
 #' 
 #' @return A validated \code{'data.frame'} based on \code{newdata}.
@@ -293,36 +386,39 @@ get_one_value_per_group <- function(x, gr) {
 #' @export
 validate_newdata <- function(
   newdata, object, re_formula = NULL, allow_new_levels = FALSE,
-  resp = NULL, check_response = TRUE, incl_autocor = TRUE,
-  all_group_vars = NULL, ...
+  newdata2 = NULL, resp = NULL, check_response = TRUE, 
+  incl_autocor = TRUE, group_vars = NULL, req_vars = NULL, ...
 ) {
-  if (is.null(newdata)) {
-    newdata <- structure(object$data, valid = TRUE, old = TRUE)
-  }
-  if (isTRUE(attr(newdata, "valid"))) {
-    return(newdata)
-  }
   newdata <- try(as.data.frame(newdata), silent = TRUE)
   if (is(newdata, "try-error")) {
     stop2("Argument 'newdata' must be coercible to a data.frame.")
   }
-  newdata <- rm_attr(newdata, c("terms", "brmsframe"))
   object <- restructure(object)
   object <- exclude_terms(object, incl_autocor = incl_autocor)
   resp <- validate_resp(resp, object)
   new_formula <- update_re_terms(formula(object), re_formula)
-  bterms <- parse_bf(new_formula, resp_rhs_all = FALSE)
+  bterms <- brmsterms(new_formula, resp_rhs_all = FALSE)
+ 
+  # fill values of not required variables
+  all_vars <- all.vars(bterms$allvars)
+  if (is.null(req_vars)) {
+    req_vars <- all_vars
+  } else {
+    req_vars <- as.character(req_vars)
+    req_vars <- intersect(req_vars, all_vars)
+  }
   if (is.mvbrmsterms(bterms) && !is.null(resp)) {
     # variables not used in the included model parts
     # do not need to be specified in newdata
     resp <- validate_resp(resp, bterms$responses)
-    reqvars <- allvars_formula(lapply(bterms$terms[resp], "[[", "allvars"))
-    not_reqvars <- setdiff(all.vars(bterms$allvars), all.vars(reqvars))
-    not_reqvars <- setdiff(not_reqvars, names(newdata))
-    if (length(not_reqvars)) {
-      newdata[, not_reqvars] <- NA
-    }
+    form_req_vars <- lapply(bterms$terms[resp], "[[", "allvars")
+    form_req_vars <- allvars_formula(form_req_vars)
+    req_vars <- intersect(req_vars, all.vars(form_req_vars))
   }
+  not_req_vars <- setdiff(all_vars, req_vars)
+  not_req_vars <- setdiff(not_req_vars, names(newdata))
+  newdata <- fill_newdata(newdata, not_req_vars, object$data)
+  # check response and addition variables
   only_resp <- all.vars(bterms$respform)
   only_resp <- setdiff(only_resp, all.vars(rhs(bterms$allvars)))
   # always require 'dec' variables to be specified
@@ -333,7 +429,7 @@ validate_newdata <- function(
       stop2("Response variables must be specified in 'newdata'.\n",
             "Missing variables: ", collapse_comma(missing_resp))
     } else {
-      newdata[, missing_resp] <- NA
+      newdata <- fill_newdata(newdata, missing_resp)
     }
   }
   # censoring and weighting vars are unused in post-processing methods
@@ -359,14 +455,16 @@ validate_newdata <- function(
     # by the user if new levels are allowed
     mis_group_vars <- new_group_vars[!grepl(":", new_group_vars)]
     mis_group_vars <- setdiff(mis_group_vars, names(newdata))
-    newdata[, mis_group_vars] <- NA
+    newdata <- fill_newdata(newdata, mis_group_vars)
   }
   newdata <- combine_groups(newdata, new_group_vars)
   # validate factor levels in newdata
-  if (is.null(all_group_vars)) {
-    all_group_vars <- get_group_vars(object) 
+  if (is.null(group_vars)) {
+    group_vars <- get_group_vars(object) 
   }
-  dont_check <- c(all_group_vars, cens_vars)
+  do_check <- union(get_pred_vars(bterms), get_int_vars(bterms))
+  dont_check <- union(group_vars, cens_vars)
+  dont_check <- setdiff(dont_check, do_check)
   dont_check <- names(mf) %in% dont_check
   is_factor <- ulapply(mf, is.factor)
   factors <- mf[is_factor & !dont_check]
@@ -378,8 +476,12 @@ validate_newdata <- function(
         if (!is.factor(new_factor)) {
           new_factor <- factor(new_factor)
         }
-        new_levels <- levels(new_factor)
         old_levels <- levels(factors[[i]])
+        if (length(old_levels) <= 1L) {
+          # contrasts are not defined for factors with 1 or fewer levels
+          next
+        }
+        new_levels <- levels(new_factor)
         old_contrasts <- contrasts(factors[[i]])
         old_ordered <- is.ordered(factors[[i]])
         to_zero <- is.na(new_factor) | new_factor %in% "zero__"
@@ -407,7 +509,7 @@ validate_newdata <- function(
   }
   # check if originally numeric variables are still numeric
   num_names <- names(mf)[!is_factor]
-  num_names <- setdiff(num_names, all_group_vars)
+  num_names <- setdiff(num_names, group_vars)
   for (nm in intersect(num_names, names(newdata))) {
     if (!anyNA(newdata[[nm]]) && !is.numeric(newdata[[nm]])) {
       stop2("Variable '", nm, "' was originally ", 
@@ -436,9 +538,7 @@ validate_newdata <- function(
   used_vars <- union(used_vars, rsv_vars(bterms))
   all_vars <- all.vars(str2formula(names(mf)))
   unused_vars <- setdiff(all_vars, used_vars)
-  if (length(unused_vars)) {
-    newdata[, unused_vars] <- NA
-  }
+  newdata <- fill_newdata(newdata, unused_vars)
   # validate grouping factors
   new_ranef <- tidy_ranef(bterms, data = mf)
   new_meef <- tidy_meef(bterms, data = mf)
@@ -460,125 +560,73 @@ validate_newdata <- function(
       }
     } 
   }
-  structure(newdata, valid = TRUE)
+  # ensure correct handling of functions like 'poly' or 'scale'
+  old_terms <- attr(object$data, "terms")
+  attr_terms <- c("variables", "predvars")
+  attr_terms <- attributes(old_terms)[attr_terms]
+  newdata <- validate_data(
+    newdata, bterms = bterms, na.action = na.pass, 
+    drop.unused.levels = FALSE, attr_terms = attr_terms,
+    data2 = current_data2(object, newdata2),
+    knots = get_knots(object$data)
+  )
+  newdata
 }
 
-# TODO: refactor preparation and storage of old standata
-# helper function for validate_newdata to extract
-# old standata required for the computation of new standata
-extract_old_standata <- function(x, data, ...) {
-  UseMethod("extract_old_standata")
-}
-
-#' @export
-extract_old_standata.default <- function(x, data, ...) {
-  NULL
-}
-
-#' @export
-extract_old_standata.mvbrmsterms <- function(x, data, ...) {
-  out <- named_list(names(x$responses))
-  for (i in seq_along(out)) {
-    out[[i]] <- extract_old_standata(x$terms[[i]], data, ...)
-  }
-  out
-}
-
-#' @export
-extract_old_standata.brmsterms <- function(x, data, ...) {
-  out <- named_list(c(names(x$dpars), names(x$nlpars)))
-  data <- subset_data(data, x)
-  for (dp in names(x$dpars)) {
-    out[[dp]] <- extract_old_standata(x$dpars[[dp]], data, ...)
-  }
-  for (nlp in names(x$nlpars)) {
-    out[[nlp]] <- extract_old_standata(x$nlpars[[nlp]], data, ...)
-  }
-  if (has_trials(x$family)) {
-    # trials should not be computed based on new data
-    datr <- data_response(x, data, check_response = FALSE, internal = TRUE)
-    # partially match via $ to be independent of the response suffix
-    out$trials <- datr$trials
-  }
-  if (is_binary(x$family) || is_categorical(x$family)) {
-    Y <- model.response(model.frame(x$respform, data, na.action = na.pass))
-    out$resp_levels <- levels(as.factor(Y))
-  }
-  if (is_cox(x$family)) {
-    # compute basis matrix of the baseline hazard for the Cox model
-    datr <- data_response(x, data, check_response = FALSE, internal = TRUE)
-    out$bhaz_basis <- bhaz_basis_matrix(datr$Y, args = x$family$bhaz)
-  }
-  out
-}
-
-#' @export
-extract_old_standata.btnl <- function(x, data, ...) {
-  NULL
-}
-
-#' @export
-extract_old_standata.btl <- function(x, data, ...) {
-  out <- list()
-  out$smooths <- make_sm_list(x, data, ...)
-  out$gps <- make_gp_list(x, data, ...)
-  out$Jmo <- make_Jmo_list(x, data, ...)
-  if (has_ac_class(x, "car")) {
-    gr <- get_ac_vars(x, "gr", class = "car")
-    stopifnot(length(gr) <= 1L)
-    if (isTRUE(nzchar(gr))) {
-      out$locations <- levels(factor(get(gr, data)))
-    } else {
-      out$locations <- NA
+# fill newdata with values for not required variables
+# @param newdata data.frame to be filled
+# @param vars character vector of not required variables
+# @param olddata optional data.frame to take values from
+# @param n row number of olddata to extract values from
+fill_newdata <- function(newdata, vars, olddata = NULL, n = 1L) {
+  stopifnot(is.data.frame(newdata), is.character(vars))
+  vars <- setdiff(vars, names(newdata))
+  if (is.null(olddata)) {
+    if (length(vars)) {
+      newdata[, vars] <- NA 
     }
+    return(newdata)
   }
-  out
-}
-
-# extract data related to smooth terms
-# for use in extract_old_standata
-# @param version optional brms version number
-make_sm_list <- function(x, data, version = NULL, ...) {
-  stopifnot(is.btl(x))
-  smterms <- all_terms(x[["sm"]])
-  out <- named_list(smterms)
-  if (length(smterms)) {
-    knots <- attr(data, "knots")
-    data <- rm_attr(data, "terms")
-    # the spline penality has changed in 2.8.7 (#646)
-    diagonal.penalty <- !isTRUE(version <= "2.8.6")
-    gam_args <- list(
-      data = data, knots = knots, 
-      absorb.cons = TRUE, modCon = 3,
-      diagonal.penalty = diagonal.penalty
-    )
-    for (i in seq_along(smterms)) {
-      sc_args <- c(list(eval2(smterms[i])), gam_args)
-      out[[i]] <- do_call(smoothCon, sc_args)
+  stopifnot(is.data.frame(olddata), length(n) == 1L)
+  for (v in vars) {
+    # using NA for variables is not safe in all cases
+    # for example when processing splines using mgcv
+    # hence it is safer to use existing data values
+    cval <- olddata[n, v] %||% NA
+    if (length(dim(cval)) == 2L) {
+      # matrix columns don't have automatic broadcasting apparently
+      cval <- matrix(cval, nrow(newdata), ncol(cval), byrow = TRUE)
     }
+    newdata[[v]] <- cval
   }
-  out
+  newdata
 }
 
-# extract data related to gaussian processes
-# for use in extract_old_standata
-make_gp_list <- function(x, data, ...) {
-  stopifnot(is.btl(x))
-  out <- data_gp(x, data, raw = TRUE)
-  out <- out[grepl("^(dmax)|(cmeans)", names(out))]
-  out
+# validate new data2 
+validate_newdata2 <- function(newdata2, object, ...) {
+  stopifnot(is.brmsfit(object))
+  bterms <- brmsterms(object$formula)
+  validate_data2(newdata2, bterms = bterms, ...)
 }
 
-# extract data related to monotonic effects
-# for use in extract_old_standata
-make_Jmo_list <- function(x, data, ...) {
-  stopifnot(is.btl(x))
-  out <- NULL
-  if (length(attr(x$sp, "uni_mo"))) {
-    # do it like data_sp()
-    spef <- tidy_spef(x, data)
-    Xmo <- lapply(unlist(spef$calls_mo), get_mo_values, data = data)
-    out <- as.array(ulapply(Xmo, max))
+# extract the current data
+current_data <- function(object, newdata = NULL, ...) {
+  stopifnot(is.brmsfit(object))
+  if (is.null(newdata)) {
+    data <- object$data
+  } else {
+    data <- validate_newdata(newdata, object = object, ...)
   }
-  out
+  data
+}
+
+# extract the current data2
+current_data2 <- function(object, newdata2 = NULL, ...) {
+  stopifnot(is.brmsfit(object))
+  if (is.null(newdata2)) {
+    data2 <- object$data2
+  } else {
+    data2 <- validate_newdata2(newdata2, object = object, ...)
+  }
+  data2
 }
