@@ -60,18 +60,20 @@ data_predictor.brmsterms <- function(x, data, data2, prior, ranef,
 #' @export
 data_predictor.btl <- function(x, data, ranef = empty_ranef(), 
                                prior = brmsprior(), data2 = list(),
-                               basis = NULL, ...) {
-  c(data_fe(x, data),
-    data_sp(x, data, data2 = data2, prior = prior, basis = basis$sp),
+                               index = NULL, basis = NULL, ...) {
+  out <- c(
+    data_fe(x, data),
+    data_sp(x, data, data2 = data2, prior = prior, index = index, basis = basis$sp),
     data_re(x, data, ranef = ranef),
     data_cs(x, data),
     data_sm(x, data, basis = basis$sm),
     data_gp(x, data, basis = basis$gp),
     data_ac(x, data, data2 = data2, basis = basis$ac),
     data_offset(x, data),
-    data_bhaz(x, data, data2 = data2, prior = prior, basis = basis$bhaz),
-    data_prior(x, data, prior = prior)
+    data_bhaz(x, data, data2 = data2, prior = prior, basis = basis$bhaz)
   )
+  c(out) <- data_prior(x, data, prior = prior, sdata = out)
+  out
 }
 
 # prepare data for non-linear parameters for use in Stan
@@ -110,7 +112,7 @@ data_sm <- function(bterms, data, basis = NULL) {
     knots <- get_knots(data)
     basis <- named_list(smterms)
     for (i in seq_along(smterms)) {
-      # the spline penality has changed in 2.8.7 (#646)
+      # the spline penalty has changed in 2.8.7 (#646)
       diagonal.penalty <- !require_old_default("2.8.7")
       basis[[i]] <- smoothCon(
         eval2(smterms[i]), data = data, 
@@ -255,8 +257,15 @@ data_gr_local <- function(bterms, data, ranef) {
         weights <- matrix(1 / ngs, nrow = nrow(data), ncol = ngs)
       }
       for (i in seq_along(gs)) {
-        J <- as.array(match(get(gs[i], data), levels))
-        out[[paste0("J_", idresp, "_", i)]] <- J
+        gdata <- get(gs[i], data)
+        J <- match(gdata, levels)
+        if (anyNA(J)) {
+          # occurs for new levels only
+          new_gdata <- gdata[!gdata %in% levels]
+          new_levels <- unique(new_gdata)
+          J[is.na(J)] <- match(new_gdata, new_levels) + length(levels)
+        }
+        out[[paste0("J_", idresp, "_", i)]] <- as.array(J)
         out[[paste0("W_", idresp, "_", i)]] <- as.array(weights[, i])
       }
     } else {
@@ -317,7 +326,7 @@ data_gr_global <- function(ranef, data2) {
 }
 
 # prepare data for special effects for use in Stan
-data_sp <- function(bterms, data, data2, prior, basis = NULL) {
+data_sp <- function(bterms, data, data2, prior, index = NULL, basis = NULL) {
   out <- list()
   spef <- tidy_spef(bterms, data)
   if (!nrow(spef)) return(out)
@@ -363,6 +372,27 @@ data_sp <- function(bterms, data, data2, prior, basis = NULL) {
       }
     }
   }
+  uni_mi <- attr(spef, "uni_mi")
+  for (j in seq_rows(uni_mi)) {
+    if (!is.na(uni_mi$idx[j])) {
+      idxl <- get(uni_mi$idx[j], data)
+      if (is.null(index[[uni_mi$var[j]]])) {
+        # the 'idx' argument needs to be mapped against 'index' addition terms
+        stop2("Response '", uni_mi$var[j], "' needs to have an 'index' addition ", 
+              "term to compare with 'idx'. See ?mi for examples.")
+      }
+      idxl <- match(idxl, index[[uni_mi$var[j]]])
+      if (anyNA(idxl)) {
+        stop2("Could not match all indices in response '", uni_mi$var[j], "'.")
+      }
+      idxl_name <- paste0("idxl", p, "_", uni_mi$var[j], "_", uni_mi$idx2[j])
+      out[[idxl_name]] <- as.array(idxl) 
+    } else if (isTRUE(attr(index[[uni_mi$var[j]]], "subset"))) {
+      # cross-formula referencing is required for subsetted variables
+      stop2("mi() terms of subsetted variables require ",
+            "the 'idx' argument to be specified.")
+    }
+  }
   out
 }
 
@@ -697,13 +727,13 @@ data_ac <- function(bterms, data, data2, basis = NULL, ...) {
         }
       }
     }
-    M_tmp <- M
-    M_tmp[upper.tri(M_tmp)] <- NA
-    edges <- which(as.matrix(M_tmp == 1), arr.ind = TRUE)
+    edges_rows <- (Matrix::tril(M)@i + 1)
+    edges_cols <- sort(Matrix::triu(M)@i + 1) ## sort to make consistent with rows
+    edges <- cbind("rows" = edges_rows, "cols" = edges_cols)
     c(out) <- nlist(
-      Nloc, Jloc, Nedges = nrow(edges),  
-      edges1 = as.array(edges[, 1]), 
-      edges2 = as.array(edges[, 2])
+      Nloc, Jloc, Nedges = length(edges_rows),
+      edges1 = as.array(edges_rows),
+      edges2 = as.array(edges_cols)
     )
     if (acef_car$type %in% c("escar", "esicar")) {
       Nneigh <- Matrix::colSums(M)
@@ -785,7 +815,7 @@ data_cnl <- function(bterms, data) {
       }
       cvalues <- cvalues[, 1]
     }
-    out[[paste0("C", p, "_", i)]] <- cvalues
+    out[[paste0("C", p, "_", i)]] <- as.array(cvalues)
   }
   out
 }
@@ -822,30 +852,41 @@ data_cnl <- function(bterms, data) {
 }
 
 # data for special priors such as horseshoe and lasso
-data_prior <- function(bterms, data, prior) {
+data_prior <- function(bterms, data, prior, sdata = NULL) {
   out <- list()
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
-  prefix <- combine_prefix(px, keep_mu = TRUE)
-  special <- attr(prior, "special")[[prefix]]
-  if (!is.null(special[["hs_df"]])) {
+  special <- get_special_prior(prior, px)
+  if (!is.null(special$horseshoe)) {
     # data for the horseshoe prior
-    hs_obj_names <- paste0("hs_", 
-      c("df", "df_global", "df_slab", "scale_global", "scale_slab")
-    )
-    hs_data <- special[hs_obj_names]
-    if (is.null(special[["hs_par_ratio"]])) {
-      hs_data$hs_scale_global <- special$hs_scale_global
-    } else {
-      hs_data$hs_scale_global <- special$hs_par_ratio / sqrt(nrow(data))
+    hs_names <- c("df", "df_global", "df_slab", "scale_global", "scale_slab")
+    hs_data <- special$horseshoe[hs_names]
+    if (!is.null(special$horseshoe$par_ratio)) {
+      hs_data$scale_global <- special$horseshoe$par_ratio / sqrt(nrow(data))
     }
-    names(hs_data) <- paste0(hs_obj_names, p) 
+    names(hs_data) <- paste0("hs_", hs_names, p) 
     out <- c(out, hs_data)
   }
-  if (!is.null(special[["lasso_df"]])) {
-    lasso_obj_names <- paste0("lasso_", c("df", "scale"))
-    lasso_data <- special[lasso_obj_names]
-    names(lasso_data) <- paste0(lasso_obj_names, p) 
+  if (!is.null(special$R2D2)) {
+    # data for the R2D2 prior
+    R2D2_names <- c("mean_R2", "prec_R2", "cons_D2")
+    R2D2_data <- special$R2D2[R2D2_names]
+    # number of coefficients minus the intercept
+    K <- sdata[[paste0("K", p)]] - ifelse(stan_center_X(bterms), 1, 0)
+    if (length(R2D2_data$cons_D2) == 1L) {
+      R2D2_data$cons_D2 <- rep(R2D2_data$cons_D2, K)
+    }
+    if (length(R2D2_data$cons_D2) != K) {
+      stop2("Argument 'cons_D2' of the R2D2 prior must be of length 1 or ", K)
+    }
+    R2D2_data$cons_D2 <- as.array(R2D2_data$cons_D2)
+    names(R2D2_data) <- paste0("R2D2_", R2D2_names, p) 
+    out <- c(out, R2D2_data)
+  }
+  if (!is.null(special$lasso)) {
+    lasso_names <- c("df", "scale")
+    lasso_data <- special$lasso[lasso_names]
+    names(lasso_data) <- paste0("lasso_", lasso_names, p) 
     out <- c(out, lasso_data)
   }
   out
@@ -899,9 +940,12 @@ smoothCon <- function(object, data, ...) {
   data <- rm_attr(data, "terms")
   vars <- setdiff(c(object$term, object$by), "NA")
   for (v in vars) {
-    # allow factor-like variables #562
     if (is_like_factor(data[[v]])) {
+      # allow factor-like variables #562
       data[[v]] <- as.factor(data[[v]])
+    } else if (inherits(data[[v]], "difftime")) {
+      # mgcv cannot handle 'difftime' variables
+      data[[v]] <- as.numeric(data[[v]])
     }
   }
   mgcv::smoothCon(object, data = data, ...)
