@@ -8,7 +8,7 @@
 #' 
 #' @param x,q Vector of quantiles.
 #' @param p Vector of probabilities.
-#' @param n Number of samples to draw from the distribution.
+#' @param n Number of draws to sample from the distribution.
 #' @param mu Vector of location values.
 #' @param sigma Vector of scale values.
 #' @param df Vector of degrees of freedom.
@@ -23,8 +23,8 @@
 #' 
 #' @export
 dstudent_t <- function(x, df, mu = 0, sigma = 1, log = FALSE) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   if (log) {
     dt((x - mu) / sigma, df = df, log = TRUE) - log(sigma)
@@ -37,8 +37,8 @@ dstudent_t <- function(x, df, mu = 0, sigma = 1, log = FALSE) {
 #' @export
 pstudent_t <- function(q, df, mu = 0, sigma = 1, 
                        lower.tail = TRUE, log.p = FALSE) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   pt((q - mu) / sigma, df = df, lower.tail = lower.tail, log.p = log.p)
 }
@@ -46,8 +46,8 @@ pstudent_t <- function(q, df, mu = 0, sigma = 1,
 #' @rdname StudentT
 #' @export
 qstudent_t <-  function(p, df, mu = 0, sigma = 1) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   mu + sigma * qt(p, df = df)
 }
@@ -55,8 +55,8 @@ qstudent_t <-  function(p, df, mu = 0, sigma = 1) {
 #' @rdname StudentT
 #' @export
 rstudent_t <- function(n, df, mu = 0, sigma = 1) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {     
+    stop2("sigma must be non-negative.")
   }
   mu + sigma * rt(n, df = df)
 }
@@ -122,8 +122,8 @@ rmulti_normal <- function(n, mu, Sigma, check = FALSE) {
       stop2("Sigma must be a symmetric matrix.")
     }
   }
-  samples <- matrix(rnorm(n * p), nrow = n, ncol = p)
-  mu + samples %*% chol(Sigma)
+  draws <- matrix(rnorm(n * p), nrow = n, ncol = p)
+  mu + draws %*% chol(Sigma)
 }
 
 #' The Multivariate Student-t Distribution
@@ -184,9 +184,9 @@ rmulti_student_t <- function(n, df, mu, Sigma, check = FALSE) {
   if (isTRUE(any(df <= 0))) {
     stop2("df must be greater than 0.")
   }
-  samples <- rmulti_normal(n, mu = rep(0, p), Sigma = Sigma, check = check)
-  samples <- samples / sqrt(rchisq(n, df = df) / df)
-  sweep(samples, 2, mu, "+")
+  draws <- rmulti_normal(n, mu = rep(0, p), Sigma = Sigma, check = check)
+  draws <- draws / sqrt(rchisq(n, df = df) / df)
+  sweep(draws, 2, mu, "+")
 }
 
 #' The Skew-Normal Distribution
@@ -215,7 +215,7 @@ rmulti_student_t <- function(n, df, mu, Sigma, check = FALSE) {
 #' @export
 dskew_normal <- function(x, mu = 0, sigma = 1, alpha = 0, 
                          xi = NULL, omega = NULL, log = FALSE) {
-  if (isTRUE(any(sigma <= 0))) {
+  if (isTRUE(any(sigma < 0))) {
     stop2("sigma must be greater than 0.")
   }
   args <- cp2dp(mu, sigma, alpha, xi = xi, omega = omega, x = x)
@@ -246,8 +246,8 @@ pskew_normal <- function(q, mu = 0, sigma = 1, alpha = 0,
                          xi = NULL, omega = NULL,
                          lower.tail = TRUE, log.p = FALSE) {
   require_package("mnormt")
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   args <- cp2dp(mu, sigma, alpha, xi = xi, omega = omega, q = q)
   out <- with(args, {
@@ -289,8 +289,8 @@ qskew_normal <- function(p, mu = 0, sigma = 1, alpha = 0,
                          xi = NULL, omega = NULL,
                          lower.tail = TRUE, log.p = FALSE, 
                          tol = 1e-8) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   if (log.p) {
     p <- exp(p)
@@ -337,8 +337,8 @@ qskew_normal <- function(p, mu = 0, sigma = 1, alpha = 0,
 #' @export
 rskew_normal <- function(n, mu = 0, sigma = 1, alpha = 0,
                          xi = NULL, omega = NULL) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
   args <- cp2dp(mu, sigma, alpha, xi = xi, omega = omega)
   with(args, {
@@ -586,11 +586,11 @@ rvon_mises <- function(n, mu, kappa) {
 #' 
 #' @export
 dexgaussian <- function(x, mu, sigma, beta, log = FALSE) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
-  if (isTRUE(any(beta <= 0))) {
-    stop2("beta must be greater than 0.")
+  if (isTRUE(any(beta < 0))) {
+    stop2("beta must be non-negative.")
   }
   args <- nlist(x, mu, sigma, beta)
   args <- do_call(expand, args)
@@ -611,11 +611,11 @@ dexgaussian <- function(x, mu, sigma, beta, log = FALSE) {
 #' @export
 pexgaussian <- function(q, mu, sigma, beta, 
                         lower.tail = TRUE, log.p = FALSE) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
-  if (isTRUE(any(beta <= 0))) {
-    stop2("beta must be greater than 0.")
+  if (isTRUE(any(beta < 0))) {
+    stop2("beta must be non-negative.")
   }
   args <- nlist(q, mu, sigma, beta)
   args <- do_call(expand, args)
@@ -639,11 +639,11 @@ pexgaussian <- function(q, mu, sigma, beta,
 #' @rdname ExGaussian
 #' @export
 rexgaussian <- function(n, mu, sigma, beta) {
-  if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma must be greater than 0.")
+  if (isTRUE(any(sigma < 0))) {
+    stop2("sigma must be non-negative.")
   }
-  if (isTRUE(any(beta <= 0))) {
-    stop2("beta must be greater than 0.")
+  if (isTRUE(any(beta < 0))) {
+    stop2("beta must be non-negative.")
   }
   mu <- mu - beta
   rnorm(n, mean = mu, sd = sigma) + rexp(n, rate = 1 / beta)
@@ -890,10 +890,9 @@ rinv_gaussian <- function(n, mu = 1, shape = 1) {
 #' on the parameterization.
 #' 
 #' @export
-dgen_extreme_value <- function(x, mu = 0, sigma = 1, 
-                               xi = 0, log = FALSE) {
+dgen_extreme_value <- function(x, mu = 0, sigma = 1, xi = 0, log = FALSE) {
   if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma bust be greater than 0.")
+    stop2("sigma bust be positive.")
   }
   x <- (x - mu) / sigma
   args <- nlist(x, mu, sigma, xi)
@@ -915,7 +914,7 @@ dgen_extreme_value <- function(x, mu = 0, sigma = 1,
 pgen_extreme_value <- function(q, mu = 0, sigma = 1, xi = 0,
                                lower.tail = TRUE, log.p = FALSE) {
   if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma bust be greater than 0.")
+    stop2("sigma bust be positive.")
   }
   q <- (q - mu) / sigma
   args <- nlist(q, mu, sigma, xi)
@@ -938,7 +937,7 @@ pgen_extreme_value <- function(q, mu = 0, sigma = 1, xi = 0,
 #' @export
 rgen_extreme_value <- function(n, mu = 0, sigma = 1, xi = 0) {
   if (isTRUE(any(sigma <= 0))) {
-    stop2("sigma bust be greater than 0.")
+    stop2("sigma bust be positive.")
   }
   args <- nlist(mu, sigma, xi, length = n)
   args <- do_call(expand, args)
@@ -1046,7 +1045,7 @@ ddiscrete_weibull <- function(x, mu, shape, log = FALSE) {
     stop2("mu bust be between 0 and 1.")
   }
   if (isTRUE(any(shape <= 0))) {
-    stop2("shape bust be greater than 0.")
+    stop2("shape bust be positive.")
   }
   x <- round(x)
   out <- mu^x^shape - mu^(x + 1)^shape
@@ -1064,7 +1063,7 @@ pdiscrete_weibull <- function(x, mu, shape, lower.tail = TRUE, log.p = FALSE) {
     stop2("mu bust be between 0 and 1.")
   }
   if (isTRUE(any(shape <= 0))) {
-    stop2("shape bust be greater than 0.")
+    stop2("shape bust be positive.")
   }
   x <- round(x)
   if (lower.tail) {
@@ -1087,7 +1086,7 @@ qdiscrete_weibull <- function(p, mu, shape, lower.tail = TRUE, log.p = FALSE) {
     stop2("mu bust be between 0 and 1.")
   }
   if (isTRUE(any(shape <= 0))) {
-    stop2("shape bust be greater than 0.")
+    stop2("shape bust be positive.")
   }
   if (log.p) {
     p <- exp(p)
@@ -1640,7 +1639,7 @@ pcox <- function(q, mu, bhaz, cbhaz, lower.tail = TRUE, log.p = FALSE) {
 #' @name ZeroInflated
 #' 
 #' @inheritParams StudentT
-#' @param zi zero-inflation propability
+#' @param zi zero-inflation probability
 #' @param mu,lambda location parameter
 #' @param shape,shape1,shape2 shape parameter
 #' @param size number of trials
@@ -1802,7 +1801,7 @@ pzero_inflated_asym_laplace <- function(q, mu, sigma, quantile, zi,
 #' @name Hurdle
 #' 
 #' @inheritParams StudentT
-#' @param hu hurdle propability
+#' @param hu hurdle probability
 #' @param mu,lambda location parameter
 #' @param shape shape parameter
 #' @param sigma,scale scale parameter
@@ -1952,7 +1951,7 @@ phurdle_lognormal <- function(q, mu, sigma, hu, lower.tail = TRUE,
 
 # density of the categorical distribution with the softmax transform
 # @param x positive integers not greater than ncat
-# @param eta the linear predictor (of length or ncol ncat-1)
+# @param eta the linear predictor (of length or ncol ncat)
 # @param log return values on the log scale?
 dcategorical <- function(x, eta, log = FALSE) {
   if (is.null(dim(eta))) {
@@ -1961,17 +1960,77 @@ dcategorical <- function(x, eta, log = FALSE) {
   if (length(dim(eta)) != 2L) {
     stop2("eta must be a numeric vector or matrix.")
   }
-  if (log) {
-    out <- log_softmax(eta)
-  } else {
-    out <- softmax(eta)
-  }
+  out <- inv_link_categorical(eta, log = log)
   out[, x, drop = FALSE]
 }
 
+# generic inverse link function for the categorical family
+# 
+# @param x Matrix (S x `ncat` or S x `ncat - 1` (depending on
+#   `insert_refcat_fam`), with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) with values of `eta` for
+#   one observation (see dcategorical()) or an array (S x N x `ncat` or S x N x
+#   `ncat - 1` (depending on `insert_refcat_fam`)) containing the same values as
+#   the matrix just described, but for N observations.
+# @param insert_refcat_fam Either NULL or an object of class "brmsfamily". If
+#   NULL, `x` is not modified at all. If an object of class "brmsfamily", then
+#   insert_refcat() is used to insert values for the reference category into
+#   `x`.
+# @param log Logical (length 1) indicating whether to log the return value.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the inverse-link function applied to
+#   `x`. If `x` is an array, then an array (S x N x `ncat`) containing the same
+#   values as the matrix just described, but for N observations.
+inv_link_categorical <- function(x, insert_refcat_fam = NULL, log = FALSE) {
+  if (!is.null(insert_refcat_fam)) {
+    x <- insert_refcat(x, family = insert_refcat_fam)
+  }
+  if (log) {
+    out <- log_softmax(x)
+  } else {
+    out <- softmax(x)
+  }
+  out
+}
+
+# generic link function for the categorical family
+# 
+# @param x Matrix (S x `ncat`, with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) of probabilities for the
+#   response categories or an array (S x N x `ncat`) containing the same values
+#   as the matrix just described, but for N observations.
+# @param refcat Numeric (length 1) giving the index of the reference category.
+# @param return_refcat Logical (length 1) indicating whether to include the
+#   reference category in the return value.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat` or S x `ncat - 1`
+#   (depending on `return_refcat`), with S denoting the number of posterior
+#   draws and `ncat` denoting the number of response categories) containing the
+#   values of the link function applied to `x`. If `x` is an array, then an
+#   array (S x N x `ncat` or S x N x `ncat - 1` (depending on `return_refcat`))
+#   containing the same values as the matrix just described, but for N
+#   observations.
+link_categorical <- function(x, refcat = 1, return_refcat = TRUE) {
+  ndim <- length(dim(x))
+  marg_noncat <- seq_along(dim(x))[-ndim]
+  if (return_refcat) {
+    x_tosweep <- x
+  } else {
+    x_tosweep <- slice(x, ndim, -refcat, drop = FALSE)
+  }
+  log(sweep(
+    x_tosweep,
+    MARGIN = marg_noncat,
+    STATS = slice(x, ndim, refcat),
+    FUN = "/"
+  ))
+}
+
 # CDF of the categorical distribution with the softmax transform
 # @param q positive integers not greater than ncat
-# @param eta the linear predictor (of length or ncol ncat-1)  
+# @param eta the linear predictor (of length or ncol ncat)  
 # @param log.p return values on the log scale?
 pcategorical <- function(q, eta, log.p = FALSE) {
   p <- dcategorical(seq_len(max(q)), eta = eta)
@@ -1984,7 +2043,7 @@ pcategorical <- function(q, eta, log.p = FALSE) {
 
 # density of the multinomial distribution with the softmax transform
 # @param x positive integers not greater than ncat
-# @param eta the linear predictor (of length or ncol ncat-1)
+# @param eta the linear predictor (of length or ncol ncat)
 # @param log return values on the log scale?
 dmultinomial <- function(x, eta, log = FALSE) {
   if (is.null(dim(eta))) {
@@ -1995,7 +2054,7 @@ dmultinomial <- function(x, eta, log = FALSE) {
   }
   log_prob <- log_softmax(eta)
   size <- sum(x)
-  x <- as_draws_matrix(x, dim = dim(eta))
+  x <- data2draws(x, dim = dim(eta))
   out <- lgamma(size + 1) + rowSums(x * log_prob - lgamma(x + 1))
   if (!log) {
     out <- exp(out)
@@ -2004,96 +2063,358 @@ dmultinomial <- function(x, eta, log = FALSE) {
 }
 
 # density of the cumulative distribution
+# 
+# @param x Integer vector containing response category indices to return the
+#   "densities" (probability masses) for.
+# @param eta Vector (length S, with S denoting the number of posterior draws) of
+#   linear predictor draws.
+# @param thres Matrix (S x `ncat - 1`, with S denoting the number of posterior
+#   draws and `ncat` denoting the number of response categories) of threshold
+#   draws.
+# @param disc Vector (length S, with S denoting the number of posterior draws,
+#   or length 1 for recycling) of discrimination parameter draws.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return A matrix (S x `length(x)`) containing the values of the inverse-link
+#   function applied to `disc * (thres - eta)`.
 dcumulative <- function(x, eta, thres, disc = 1, link = "logit") {
-  eta <- ilink(disc * (thres - eta), link)
-  ncat <- ncol(eta) + 1
-  rows <- list(eta[, 1])
-  if (ncat > 2) {
-    .fun <- function(k) {
-      eta[, k] - eta[, k - 1]
-    }
-    rows <- c(rows, lapply(2:(ncat - 1), .fun))
+  eta <- disc * (thres - eta)
+  if (link == "identity") {
+    out <- eta
+  } else {
+    out <- inv_link_cumulative(eta, link = link)
   }
-  rows <- c(rows, list(1 - eta[, ncat - 1]))
-  p <- do_call(cbind, rows)
-  p[, x, drop = FALSE]
+  out[, x, drop = FALSE] 
+}
+
+# generic inverse link function for the cumulative family
+# 
+# @param x Matrix (S x `ncat - 1`, with S denoting the number of posterior draws
+#   and `ncat` denoting the number of response categories) with values of
+#   `disc * (thres - eta)` for one observation (see dcumulative()) or an array
+#   (S x N x `ncat - 1`) containing the same values as the matrix just
+#   described, but for N observations.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the inverse-link function applied to
+#   `x`. If `x` is an array, then an array (S x N x `ncat`) containing the same
+#   values as the matrix just described, but for N observations.
+inv_link_cumulative <- function(x, link) {
+  x <- ilink(x, link)
+  ndim <- length(dim(x))
+  dim_noncat <- dim(x)[-ndim]
+  ones_arr <- array(1, dim = c(dim_noncat, 1))
+  zeros_arr <- array(0, dim = c(dim_noncat, 1))
+  abind::abind(x, ones_arr) - abind::abind(zeros_arr, x)
+}
+
+# generic link function for the cumulative family
+# 
+# @param x Matrix (S x `ncat`, with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) of probabilities for the
+#   response categories or an array (S x N x `ncat`) containing the same values
+#   as the matrix just described, but for N observations.
+# @param link Character string (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat - 1`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the link function applied to `x`. If
+#   `x` is an array, then an array (S x N x `ncat - 1`) containing the same
+#   values as the matrix just described, but for N observations.
+link_cumulative <- function(x, link) {
+  ndim <- length(dim(x))
+  ncat <- dim(x)[ndim]
+  dim_noncat <- dim(x)[-ndim]
+  nthres <- dim(x)[ndim] - 1
+  marg_noncat <- seq_along(dim(x))[-ndim]
+  dim_t <- c(nthres, dim_noncat)
+  x <- apply(slice(x, ndim, -ncat, drop = FALSE), marg_noncat, cumsum)
+  x <- aperm(array(x, dim = dim_t), perm = c(marg_noncat + 1, 1))
+  link(x, link)
 }
 
 # density of the sratio distribution
+# 
+# @param x Integer vector containing response category indices to return the
+#   "densities" (probability masses) for.
+# @param eta Vector (length S, with S denoting the number of posterior draws) of
+#   linear predictor draws.
+# @param thres Matrix (S x `ncat - 1`, with S denoting the number of posterior
+#   draws and `ncat` denoting the number of response categories) of threshold
+#   draws.
+# @param disc Vector (length S, with S denoting the number of posterior draws,
+#   or length 1 for recycling) of discrimination parameter draws.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return A matrix (S x `length(x)`) containing the values of the inverse-link
+#   function applied to `disc * (thres - eta)`.
 dsratio <- function(x, eta, thres, disc = 1, link = "logit") {
-  eta <- ilink(disc * (thres - eta), link)
-  ncat <- ncol(eta) + 1
-  rows <- list(eta[, 1])
-  if (ncat > 2) {
-    .fun <- function(k) {
-      (eta[, k]) * apply(as.matrix(1 - eta[, 1:(k - 1)]), 1, prod)
-    }
-    rows <- c(rows, lapply(2:(ncat - 1), .fun))
+  eta <- disc * (thres - eta)
+  if (link == "identity") {
+    out <- eta
+  } else {
+    out <- inv_link_sratio(eta, link = link)
   }
-  rows <- c(rows, list(apply(1 - eta, 1, prod)))
-  p <- do_call(cbind, rows)
-  p[, x, drop = FALSE]
+  out[, x, drop = FALSE]
+}
+
+# generic inverse link function for the sratio family
+# 
+# @param x Matrix (S x `ncat - 1`, with S denoting the number of posterior draws
+#   and `ncat` denoting the number of response categories) with values of
+#   `disc * (thres - eta)` for one observation (see dsratio()) or an array
+#   (S x N x `ncat - 1`) containing the same values as the matrix just
+#   described, but for N observations.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the inverse-link function applied to
+#   `x`. If `x` is an array, then an array (S x N x `ncat`) containing the same
+#   values as the matrix just described, but for N observations.
+inv_link_sratio <- function(x, link) {
+  x <- ilink(x, link)
+  ndim <- length(dim(x))
+  dim_noncat <- dim(x)[-ndim]
+  nthres <- dim(x)[ndim]
+  marg_noncat <- seq_along(dim(x))[-ndim]
+  ones_arr <- array(1, dim = c(dim_noncat, 1))
+  dim_t <- c(nthres, dim_noncat)
+  Sx_cumprod <- aperm(
+    array(apply(1 - x, marg_noncat, cumprod), dim = dim_t),
+    perm = c(marg_noncat + 1, 1)
+  )
+  abind::abind(x, ones_arr) * abind::abind(ones_arr, Sx_cumprod)
+}
+
+# generic link function for the sratio family
+# 
+# @param x Matrix (S x `ncat`, with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) of probabilities for the
+#   response categories or an array (S x N x `ncat`) containing the same values
+#   as the matrix just described, but for N observations.
+# @param link Character string (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat - 1`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the link function applied to `x`. If
+#   `x` is an array, then an array (S x N x `ncat - 1`) containing the same
+#   values as the matrix just described, but for N observations.
+link_sratio <- function(x, link) {
+  ndim <- length(dim(x))
+  .F_k <- function(k) {
+    if (k == 1) {
+      prev_res <- list(F_k = NULL, S_km1_prod = 1)
+    } else {
+      prev_res <- .F_k(k - 1)
+    }
+    F_k <- slice(x, ndim, k, drop = FALSE) / prev_res$S_km1_prod
+    .out <- list(
+      F_k = abind::abind(prev_res$F_k, F_k),
+      S_km1_prod = prev_res$S_km1_prod * (1 - F_k)
+    )
+    return(.out)
+  }
+  x <- .F_k(dim(x)[ndim] - 1)$F_k
+  link(x, link)
 }
 
 # density of the cratio distribution
+# 
+# @param x Integer vector containing response category indices to return the
+#   "densities" (probability masses) for.
+# @param eta Vector (length S, with S denoting the number of posterior draws) of
+#   linear predictor draws.
+# @param thres Matrix (S x `ncat - 1`, with S denoting the number of posterior
+#   draws and `ncat` denoting the number of response categories) of threshold
+#   draws.
+# @param disc Vector (length S, with S denoting the number of posterior draws,
+#   or length 1 for recycling) of discrimination parameter draws.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return A matrix (S x `length(x)`) containing the values of the inverse-link
+#   function applied to `disc * (thres - eta)`.
 dcratio <- function(x, eta, thres, disc = 1, link = "logit") {
-  eta <- ilink(disc * (eta - thres), link)
-  ncat <- ncol(eta) + 1
-  rows <- list(1 - eta[, 1])
-  if (ncat > 2) {
-    .fun <- function(k) {
-      (1 - eta[, k]) * apply(as.matrix(eta[, 1:(k - 1)]), 1, prod)
-    }
-    rows <- c(rows, lapply(2:(ncat - 1), .fun))
+  eta <- disc * (eta - thres)
+  if (link == "identity") {
+    out <- eta
+  } else {
+    out <- inv_link_cratio(eta, link = link)
   }
-  rows <- c(rows, list(apply(eta, 1, prod)))
-  p <- do_call(cbind, rows)
-  p[, x, drop = FALSE]
+  out[, x, drop = FALSE]
+}
+
+# generic inverse link function for the cratio family
+# 
+# @param x Matrix (S x `ncat - 1`, with S denoting the number of posterior draws
+#   and `ncat` denoting the number of response categories) with values of
+#   `disc * (thres - eta)` for one observation (see dcratio()) or an array
+#   (S x N x `ncat - 1`) containing the same values as the matrix just
+#   described, but for N observations.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the inverse-link function applied to
+#   `x`. If `x` is an array, then an array (S x N x `ncat`) containing the same
+#   values as the matrix just described, but for N observations.
+inv_link_cratio <- function(x, link) {
+  x <- ilink(x, link)
+  ndim <- length(dim(x))
+  dim_noncat <- dim(x)[-ndim]
+  nthres <- dim(x)[ndim]
+  marg_noncat <- seq_along(dim(x))[-ndim]
+  ones_arr <- array(1, dim = c(dim_noncat, 1))
+  dim_t <- c(nthres, dim_noncat)
+  x_cumprod <- aperm(
+    array(apply(x, marg_noncat, cumprod), dim = dim_t),
+    perm = c(marg_noncat + 1, 1)
+  )
+  abind::abind(1 - x, ones_arr) * abind::abind(ones_arr, x_cumprod)
+}
+
+# generic link function for the cratio family
+# 
+# @param x Matrix (S x `ncat`, with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) of probabilities for the
+#   response categories or an array (S x N x `ncat`) containing the same values
+#   as the matrix just described, but for N observations.
+# @param link Character string (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat - 1`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the link function applied to `x`. If
+#   `x` is an array, then an array (S x N x `ncat - 1`) containing the same
+#   values as the matrix just described, but for N observations.
+link_cratio <- function(x, link) {
+  ndim <- length(dim(x))
+  .F_k <- function(k) {
+    if (k == 1) {
+      prev_res <- list(F_k = NULL, F_km1_prod = 1)
+    } else {
+      prev_res <- .F_k(k - 1)
+    }
+    F_k <- 1 - slice(x, ndim, k, drop = FALSE) / prev_res$F_km1_prod
+    .out <- list(
+      F_k = abind::abind(prev_res$F_k, F_k),
+      F_km1_prod = prev_res$F_km1_prod * F_k
+    )
+    return(.out)
+  }
+  x <- .F_k(dim(x)[ndim] - 1)$F_k
+  link(x, link)
 }
 
 # density of the acat distribution
+# 
+# @param x Integer vector containing response category indices to return the
+#   "densities" (probability masses) for.
+# @param eta Vector (length S, with S denoting the number of posterior draws) of
+#   linear predictor draws.
+# @param thres Matrix (S x `ncat - 1`, with S denoting the number of posterior
+#   draws and `ncat` denoting the number of response categories) of threshold
+#   draws.
+# @param disc Vector (length S, with S denoting the number of posterior draws,
+#   or length 1 for recycling) of discrimination parameter draws.
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return A matrix (S x `length(x)`) containing the values of the inverse-link
+#   function applied to `disc * (thres - eta)`.
 dacat <- function(x, eta, thres, disc = 1, link = "logit") {
   eta <- disc * (eta - thres)
-  ncat <- ncol(eta) + 1
+  if (link == "identity") {
+    out <- eta
+  } else {
+    out <- inv_link_acat(eta, link = link)
+  }
+  out[, x, drop = FALSE]
+}
+
+# generic inverse link function for the acat family
+# 
+# @param x Matrix (S x `ncat - 1`, with S denoting the number of posterior draws
+#   and `ncat` denoting the number of response categories) with values of
+#   `disc * (thres - eta)` (see dacat()).
+# @param link Character vector (length 1) giving the name of the link function.
+# 
+# @return A matrix (S x `ncat`, with S denoting the number of posterior draws
+#   and `ncat` denoting the number of response categories) containing the values
+#   of the inverse-link function applied to `x`.
+inv_link_acat <- function(x, link) {
+  ndim <- length(dim(x))
+  dim_noncat <- dim(x)[-ndim]
+  nthres <- dim(x)[ndim]
+  marg_noncat <- seq_along(dim(x))[-ndim]
+  ones_arr <- array(1, dim = c(dim_noncat, 1))
+  dim_t <- c(nthres, dim_noncat)
   if (link == "logit") { 
     # faster evaluation in this case
-    p <- cbind(
-      rep(1, nrow(eta)), exp(eta[, 1]), 
-      matrix(NA, nrow = nrow(eta), ncol = ncat - 2)
+    exp_x_cumprod <- aperm(
+      array(apply(exp(x), marg_noncat, cumprod), dim = dim_t),
+      perm = c(marg_noncat + 1, 1)
     )
-    if (ncat > 2) {
-      .fun <- function(k) {
-        rowSums(eta[, 1:(k - 1)])
-      }
-      p[, 3:ncat] <- exp(sapply(3:ncat, .fun))
-    }
+    out <- abind::abind(ones_arr, exp_x_cumprod)
   } else {
-    eta <- ilink(eta, link)
-    p <- cbind(
-      apply(1 - eta[, 1:(ncat - 1)], 1, prod), 
-      matrix(0, nrow = nrow(eta), ncol = ncat - 1)
+    x <- ilink(x, link)
+    x_cumprod <- aperm(
+      array(apply(x, marg_noncat, cumprod), dim = dim_t),
+      perm = c(marg_noncat + 1, 1)
     )
-    if (ncat > 2) {
-      .fun <- function(k) {
-        apply(as.matrix(eta[, 1:(k - 1)]), 1, prod) * 
-          apply(as.matrix(1 - eta[, k:(ncat - 1)]), 1, prod)
-      }
-      p[, 2:(ncat - 1)] <- sapply(2:(ncat - 1), .fun)
-    }
-    p[, ncat] <- apply(eta[, 1:(ncat - 1)], 1, prod)
+    Sx_cumprod_rev <- apply(
+      1 - slice(x, ndim, rev(seq_len(nthres)), drop = FALSE),
+      marg_noncat, cumprod
+    )
+    Sx_cumprod_rev <- aperm(
+      array(Sx_cumprod_rev, dim = dim_t), 
+      perm = c(marg_noncat + 1, 1)
+    )
+    Sx_cumprod_rev <- slice(
+      Sx_cumprod_rev, ndim, rev(seq_len(nthres)), drop = FALSE
+    )
+    out <- abind::abind(ones_arr, x_cumprod) *
+      abind::abind(Sx_cumprod_rev, ones_arr)
   }
-  p <- p / rowSums(p)
-  p[, x, drop = FALSE]
+  catsum <- array(apply(out, marg_noncat, sum), dim = dim_noncat)
+  sweep(out, marg_noncat, catsum, "/")
+}
+
+# generic link function for the acat family
+# 
+# @param x Matrix (S x `ncat`, with S denoting the number of posterior draws and
+#   `ncat` denoting the number of response categories) of probabilities for the
+#   response categories or an array (S x N x `ncat`) containing the same values
+#   as the matrix just described, but for N observations.
+# @param link Character string (length 1) giving the name of the link function.
+# 
+# @return If `x` is a matrix, then a matrix (S x `ncat - 1`, with S denoting the
+#   number of posterior draws and `ncat` denoting the number of response
+#   categories) containing the values of the link function applied to `x`. If
+#   `x` is an array, then an array (S x N x `ncat - 1`) containing the same
+#   values as the matrix just described, but for N observations.
+link_acat <- function(x, link) {
+  ndim <- length(dim(x))
+  ncat <- dim(x)[ndim]
+  x <- slice(x, ndim, -1, drop = FALSE) / slice(x, ndim, -ncat, drop = FALSE)
+  if (link == "logit") {
+    # faster evaluation in this case
+    out <- log(x)
+  } else {
+    x <- inv_odds(x)
+    out <- link(x, link)
+  }
+  out
 }
 
 # CDF for ordinal distributions
 # @param q positive integers not greater than ncat
-# @param eta samples of the linear predictor
-# @param thres samples of threshold parameters
-# @param disc samples of the discrimination parameter
+# @param eta draws of the linear predictor
+# @param thres draws of threshold parameters
+# @param disc draws of the discrimination parameter
 # @param family a character string naming the family
 # @param link a character string naming the link
-# @return a matrix of probabilites P(x <= q)
+# @return a matrix of probabilities P(x <= q)
 pordinal <- function(q, eta, thres, disc = 1, family = NULL, link = "logit") {
   family <- as_one_character(family)
   link <- as_one_character(link)
