@@ -268,7 +268,7 @@ sar <- function(M, type = "lag") {
 #' 
 #' @param M Adjacency matrix of locations. All non-zero entries are treated as
 #'   if the two locations are adjacent. If \code{gr} is specified, the row names
-#'   of \code{W} have to match the levels of the grouping factor.
+#'   of \code{M} have to match the levels of the grouping factor.
 #' @param gr An optional grouping factor mapping observations to spatial
 #'   locations. If not specified, each observation is treated as a separate
 #'   location. It is recommended to always specify a grouping factor to allow
@@ -578,14 +578,10 @@ use_ac_cov_time <- function(x) {
   has_ac_subset(x, cov = TRUE, dim = "time")
 }
 
-# should natural residuals be modeled as correlated?
-has_cor_natural_residuals <- function(bterms) {
-  has_natural_residuals(bterms) && use_ac_cov(bterms)
-}
-
-# has the model correlated latent residuals
-has_cor_latent_residuals <- function(bterms) {
-  !has_natural_residuals(bterms) && use_ac_cov(bterms)
+# does the model need latent residuals for autocor structures?
+has_ac_latent_residuals <- function(bterms) {
+  !has_natural_residuals(bterms) &&
+    (use_ac_cov(bterms) || has_ac_class(bterms, "arma"))
 }
 
 # validate SAR matrices
@@ -614,10 +610,10 @@ validate_car_matrix <- function(M) {
     stop2("'M' for CAR terms must be symmetric.")
   }
   colnames(M) <- rownames(M)
-  not_binary <- !(M == 0 | M == 1)
+  not_binary <- M@x != 1
   if (any(not_binary)) {
     message("Converting all non-zero values in 'M' to 1.")
-    M[not_binary] <- 1
+    M@x[not_binary] <- 1
   }
   M
 }
@@ -642,7 +638,8 @@ validate_fcor_matrix <- function(M) {
 
 # regex to extract all parameter names of autocorrelation structures
 regex_autocor_pars <- function() {
-  p <- c("ar", "ma", "sderr", "cosy", "lagsar", "errorsar", "car", "sdcar")
+  p <- c("ar", "ma", "sderr", "cosy", "lagsar", "errorsar", 
+         "car", "sdcar", "rhocar")
   p <- paste0("(", p, ")", collapse = "|")
   paste0("^(", p, ")(\\[|_|$)")
 }
