@@ -15,7 +15,8 @@
 #'   \code{FALSE} and \code{TRUE}. \code{resp_cens} requires \code{'left'},
 #'   \code{'none'}, \code{'right'}, and \code{'interval'} (or equivalently
 #'   \code{-1}, \code{0}, \code{1}, and \code{2}) to indicate left, no, right,
-#'   or interval censoring.
+#'   or interval censoring. \code{resp_index} does not make any requirements
+#'   other than the value being unique for each observation.
 #' @param sigma Logical; Indicates whether the residual standard deviation
 #'  parameter \code{sigma} should be included in addition to the known
 #'  measurement error. Defaults to \code{FALSE} for backwards compatibility,
@@ -38,7 +39,9 @@
 #'   the denominator values from which the response rates are computed.
 #' @param gr A vector of grouping indicators.
 #' @param ... For \code{resp_vreal}, vectors of real values. 
-#'   For \code{resp_vint}, vectors of integer values.  
+#'   For \code{resp_vint}, vectors of integer values. In Stan,
+#'   these variables will be named \code{vreal1}, \code{vreal2}, ...,
+#'   and \code{vint1}, \code{vint2}, ..., respectively.
 #'
 #' @return A list of additional response information to be processed further
 #'   by \pkg{brms}.
@@ -169,6 +172,13 @@ resp_mi <- function(sdy = NA) {
   class_resp_special("mi", call = match.call(), vars = nlist(sdy))
 }
 
+#' @rdname addition-terms
+#' @export
+resp_index <- function(x) {
+  index <- deparse(substitute(x))
+  class_resp_special("index", call = match.call(), vars = nlist(index))
+}
+
 #' @rdname addition-terms
 #' @export
 resp_rate <- function(denom) {
@@ -357,3 +367,60 @@ has_subset <- function(bterms) {
   }
   out 
 }
+
+# construct a list of indices for cross-formula referencing
+tidy_index <- function(x, data) {
+  out <- .tidy_index(x, data)
+  if (is.brmsterms(x)) {
+    # ensure consistent format for both uni- and multivariate models
+    out <- list(out)
+    names(out)[1] <- terms_resp(x$respform) 
+  }
+  out
+}
+
+# internal version of tidy_index
+.tidy_index <- function(x, ...) {
+  UseMethod(".tidy_index")
+}
+
+#' @export
+.tidy_index.brmsterms <- function(x, data, ...) {
+  out <- get_ad_values(x, "index", "index", data)
+  if (is.null(out)) {
+    return(NULL)
+  }
+  if (has_subset(x)) {
+    subset <- as.logical(get_ad_values(x, "subset", "subset", data))
+    out <- out[subset]
+    attr(out, "subset") <- TRUE
+  }
+  if (anyNA(out)) {
+    stop2("NAs are not allowed in 'index' variables.")
+  }
+  if (anyDuplicated(out)) {
+    stop2("Index of response '", names(out), "' contains duplicated values.")
+  }
+  out
+}
+
+#' @export
+.tidy_index.mvbrmsterms <- function(x, data, ...) {
+  lapply(x$terms, .tidy_index, data = data, ...)
+}
+
+# check if cross-formula referencing is possible in subsetted models
+check_cross_formula_indexing <- function(bterms) {
+  sp_terms <- ulapply(get_effect(bterms, "sp"), all_terms)
+  me_terms <- get_matches_expr(regex_sp("me"), sp_terms)
+  if (length(me_terms)) {
+    stop2("Cannot use me() terms in subsetted formulas.")
+  }
+  mi_terms <- get_matches_expr(regex_sp("mi"), sp_terms)
+  idx_vars <- lapply(mi_terms, function(x) eval2(x)$idx)
+  if (any(idx_vars == "NA")) {
+    stop2("mi() terms in subsetted formulas require ",
+          "the 'idx' argument to be specified.")
+  }
+  invisible(TRUE)
+}
