@@ -237,10 +237,14 @@ kfold.brmsfit <- function(x, ..., K = 10, Ksub = NULL, folds = NULL,
     fits <- array(list(), dim = c(length(Ksub), 3))
     dimnames(fits) <- list(NULL, c("fit", "omitted", "predicted"))
   }
+  
+  x <- recompile_model(x)
   for (k in Ksub) {
     ks <- match(k, Ksub)
     message("Fitting model ", k, " out of ", K)
-    futures[[ks]] <- future::future(.kfold_k(k), packages = "brms")
+    futures[[ks]] <- future::future(
+      .kfold_k(k), packages = "brms", seed = TRUE
+    )
   }
   for (k in Ksub) {
     ks <- match(k, Ksub)
@@ -334,9 +338,9 @@ kfold_predict <- function(x, method = c("predict", "fitted"),
   resp <- validate_resp(resp, x$fits[[1, "fit"]], multiple = FALSE)
   all_predicted <- as.character(sort(unlist(x$fits[, "predicted"])))
   npredicted <- length(all_predicted)
-  nsamples <- nsamples(x$fits[[1, "fit"]])
+  ndraws <- ndraws(x$fits[[1, "fit"]])
   y <- rep(NA, npredicted)
-  yrep <- matrix(NA, nrow = nsamples, ncol = npredicted)
+  yrep <- matrix(NA, nrow = ndraws, ncol = npredicted)
   names(y) <- colnames(yrep) <- all_predicted
   for (k in seq_rows(x$fits)) {
     fit_k <- x$fits[[k, "fit"]]
