@@ -17,12 +17,13 @@
 #'
 #' @export
 make_stancode <- function(formula, data, family = gaussian(), 
-                          prior = NULL, autocor = NULL,
+                          prior = NULL, autocor = NULL, data2 = NULL,
                           cov_ranef = NULL, sparse = NULL, 
                           sample_prior = "no", stanvars = NULL, 
                           stan_funs = NULL, knots = NULL, 
-                          threads = NULL, save_model = NULL, 
-                          ...) {
+                          threads = NULL, 
+                          normalize = getOption("brms.normalize", TRUE),
+                          save_model = NULL, ...) {
   
   if (is.brmsfit(formula)) {
     stop2("Use 'stancode' to extract Stan code from 'brmsfit' objects.")
@@ -33,8 +34,16 @@ make_stancode <- function(formula, data, family = gaussian(),
     cov_ranef = cov_ranef
   )
   bterms <- brmsterms(formula)
-  data <- validate_data(data, bterms = bterms, knots = knots)
-  prior <- validate_prior(
+  data2 <- validate_data2(
+    data2, bterms = bterms, 
+    get_data2_autocor(formula),
+    get_data2_cov_ranef(formula)
+  )
+  data <- validate_data(
+    data, bterms = bterms, 
+    data2 = data2, knots = knots
+  )
+  prior <- .validate_prior(
     prior, bterms = bterms, data = data,
     sample_prior = sample_prior
   )
@@ -44,7 +53,7 @@ make_stancode <- function(formula, data, family = gaussian(),
  .make_stancode(
    bterms, data = data, prior = prior, 
    stanvars = stanvars, threads = threads,
-   save_model = save_model,
+   normalize = normalize, save_model = save_model,
    ...
  ) 
 }
@@ -55,22 +64,27 @@ make_stancode <- function(formula, data, family = gaussian(),
 # @param silent silence parsing messages
 .make_stancode <- function(bterms, data, prior, stanvars, 
                            threads = threading(), 
+                           normalize = getOption("brms.normalize", TRUE),
                            parse = getOption("brms.parse_stancode", FALSE), 
                            backend = getOption("brms.backend", "rstan"),
                            silent = TRUE, save_model = NULL, ...) {
- 
+  normalize <- as_one_logical(normalize)
   parse <- as_one_logical(parse)
   backend <- match.arg(backend, backend_choices())
   silent <- as_one_logical(silent)
   ranef <- tidy_ranef(bterms, data = data)
   meef <- tidy_meef(bterms, data = data)
   scode_predictor <- stan_predictor(
-    bterms, data = data, prior = prior, 
-    ranef = ranef, meef = meef,
+    bterms, data = data, prior = prior,
+    normalize = normalize, ranef = ranef, meef = meef,
     stanvars = stanvars, threads = threads
   )
-  scode_ranef <- stan_re(ranef, prior = prior, threads = threads)
-  scode_Xme <- stan_Xme(meef, prior = prior, threads = threads)
+  scode_ranef <- stan_re(
+    ranef, prior = prior, threads = threads, normalize = normalize
+  )
+  scode_Xme <- stan_Xme(
+    meef, prior = prior, threads = threads, normalize = normalize
+  )
   scode_global_defs <- stan_global_defs(
     bterms, prior = prior, ranef = ranef, threads = threads
   )
@@ -81,28 +95,31 @@ make_stancode <- function(formula, data, family = gaussian(),
     for (i in seq_along(scode_predictor)) {
       resp <- usc(names(scode_predictor)[i])
       pll_args <- stan_clean_pll_args(
-        scode_predictor[[i]]$pll_args,
-        scode_ranef$pll_args,
-        scode_Xme$pll_args
+        scode_predictor[[i]][["pll_args"]],
+        scode_ranef[["pll_args"]],
+        scode_Xme[["pll_args"]],
+        collapse_stanvars_pll_args(stanvars)
       )
       partial_log_lik <- paste0(
-        scode_predictor[[i]]$pll_def,
-        scode_predictor[[i]]$model_def,
-        scode_predictor[[i]]$model_comp_basic,
-        scode_predictor[[i]]$model_comp_eta_loop,
-        scode_predictor[[i]]$model_comp_dpar_link,
-        scode_predictor[[i]]$model_comp_mu_link,
-        scode_predictor[[i]]$model_comp_dpar_trans,
-        scode_predictor[[i]]$model_comp_mix,
-        scode_predictor[[i]]$model_comp_arma,
-        scode_predictor[[i]]$model_comp_catjoin,
-        scode_predictor[[i]]$model_comp_mvjoin,
-        scode_predictor[[i]]$model_log_lik
+        scode_predictor[[i]][["pll_def"]],
+        scode_predictor[[i]][["model_def"]],
+        collapse_stanvars(stanvars, "likelihood", "start"),
+        scode_predictor[[i]][["model_comp_basic"]],
+        scode_predictor[[i]][["model_comp_eta_loop"]],
+        scode_predictor[[i]][["model_comp_dpar_link"]],
+        scode_predictor[[i]][["model_comp_mu_link"]],
+        scode_predictor[[i]][["model_comp_dpar_trans"]],
+        scode_predictor[[i]][["model_comp_mix"]],
+        scode_predictor[[i]][["model_comp_arma"]],
+        scode_predictor[[i]][["model_comp_catjoin"]],
+        scode_predictor[[i]][["model_comp_mvjoin"]],
+        scode_predictor[[i]][["model_log_lik"]],
+        collapse_stanvars(stanvars, "likelihood", "end")
       )
       partial_log_lik <- gsub(" target \\+=", " ptarget +=", partial_log_lik)
       partial_log_lik <- paste0(
         "// compute partial sums of the log-likelihood\n",
-        "real partial_log_lik", resp, "(int[] seq", resp, 
+        "real partial_log_lik_lpmf", resp, "(int[] seq", resp, 
         ", int start, int end", pll_args$typed, ") {\n",
         "  real ptarget = 0;\n",
         "  int N = end - start + 1;\n",
@@ -111,52 +128,55 @@ make_stancode <- function(formula, data, family = gaussian(),
         "}\n"
       )
       partial_log_lik <- wsp_per_line(partial_log_lik, 2)
-      scode_predictor[[i]]$partial_log_lik <- partial_log_lik
+      scode_predictor[[i]][["partial_log_lik"]] <- partial_log_lik
       static <- str_if(threads$static, "_static")
-      scode_predictor[[i]]$model_lik <- paste0(
-        "  target += reduce_sum", static, "(partial_log_lik", resp, 
+      scode_predictor[[i]][["model_lik"]] <- paste0(
+        "  target += reduce_sum", static, "(partial_log_lik_lpmf", resp, 
         ", seq", resp, ", grainsize", pll_args$plain, ");\n"
       )
-      str_add(scode_predictor[[i]]$tdata_def) <- glue(
+      str_add(scode_predictor[[i]][["tdata_def"]]) <- glue(
         "  int seq{resp}[N{resp}] = sequence(1, N{resp});\n"
       )
     }
     scode_predictor <- collapse_lists(ls = scode_predictor)
-    scode_predictor$model_lik <- paste0(
-      scode_predictor$model_no_pll_def,
-      scode_predictor$model_no_pll_comp_basic,
-      scode_predictor$model_no_pll_comp_mvjoin,
-      scode_predictor$model_lik
+    scode_predictor[["model_lik"]] <- paste0(
+      scode_predictor[["model_no_pll_def"]],
+      scode_predictor[["model_no_pll_comp_basic"]],
+      scode_predictor[["model_no_pll_comp_mvjoin"]],
+      scode_predictor[["model_lik"]]
     )
-    str_add(scode_predictor$data) <- 
+    str_add(scode_predictor[["data"]]) <- 
       "  int grainsize;  // grainsize for threading\n" 
   } else {
     # threading is not activated
     scode_predictor <- collapse_lists(ls = scode_predictor)
-    scode_predictor$model_lik <- paste0(
-      scode_predictor$model_no_pll_def,
-      scode_predictor$model_def,
-      scode_predictor$model_no_pll_comp_basic,
-      scode_predictor$model_comp_basic,
-      scode_predictor$model_comp_eta_loop,
-      scode_predictor$model_comp_dpar_link,
-      scode_predictor$model_comp_mu_link,
-      scode_predictor$model_comp_dpar_trans,
-      scode_predictor$model_comp_mix,
-      scode_predictor$model_comp_arma,
-      scode_predictor$model_comp_catjoin,
-      scode_predictor$model_no_pll_comp_mvjoin,
-      scode_predictor$model_comp_mvjoin,
-      scode_predictor$model_log_lik
+    scode_predictor[["model_lik"]] <- paste0(
+      scode_predictor[["model_no_pll_def"]],
+      scode_predictor[["model_def"]],
+      collapse_stanvars(stanvars, "likelihood", "start"),
+      scode_predictor[["model_no_pll_comp_basic"]],
+      scode_predictor[["model_comp_basic"]],
+      scode_predictor[["model_comp_eta_loop"]],
+      scode_predictor[["model_comp_dpar_link"]],
+      scode_predictor[["model_comp_mu_link"]],
+      scode_predictor[["model_comp_dpar_trans"]],
+      scode_predictor[["model_comp_mix"]],
+      scode_predictor[["model_comp_arma"]],
+      scode_predictor[["model_comp_catjoin"]],
+      scode_predictor[["model_no_pll_comp_mvjoin"]],
+      scode_predictor[["model_comp_mvjoin"]],
+      scode_predictor[["model_log_lik"]],
+      collapse_stanvars(stanvars, "likelihood", "end")
     )
   }
-  scode_predictor$model_lik <- wsp_per_line(scode_predictor$model_lik, 2)
+  scode_predictor[["model_lik"]] <- 
+    wsp_per_line(scode_predictor[["model_lik"]], 2)
     
   # get priors for all parameters in the model
   scode_prior <- paste0(
-    scode_predictor$prior,
-    scode_ranef$prior,
-    scode_Xme$prior,
+    scode_predictor[["prior"]],
+    scode_ranef[["prior"]],
+    scode_Xme[["prior"]],
     stan_unchecked_prior(prior)
   )
   
@@ -164,9 +184,9 @@ make_stancode <- function(formula, data, family = gaussian(),
   scode_functions <- paste0(
     "// generated with brms ", utils::packageVersion("brms"), "\n",
     "functions {\n",
-      scode_global_defs$fun,
-      scode_predictor$partial_log_lik,
+      scode_global_defs[["fun"]],
       collapse_stanvars(stanvars, "functions"),
+      scode_predictor[["partial_log_lik"]],
     "}\n"
   )
   
@@ -174,9 +194,9 @@ make_stancode <- function(formula, data, family = gaussian(),
   scode_data <- paste0(
     "data {\n",
     "  int<lower=1> N;  // total number of observations\n",
-    scode_predictor$data,
-    scode_ranef$data,
-    scode_Xme$data,
+    scode_predictor[["data"]],
+    scode_ranef[["data"]],
+    scode_Xme[["data"]],
     "  int prior_only;  // should the likelihood be ignored?\n",
     collapse_stanvars(stanvars, "data"),
     "}\n"
@@ -185,32 +205,32 @@ make_stancode <- function(formula, data, family = gaussian(),
   # generate transformed parameters block
   scode_transformed_data <- paste0(
     "transformed data {\n",
-       scode_global_defs$tdata_def,
-       scode_predictor$tdata_def,
+       scode_global_defs[["tdata_def"]],
+       scode_predictor[["tdata_def"]],
        collapse_stanvars(stanvars, "tdata", "start"),
-       scode_predictor$tdata_comp,
+       scode_predictor[["tdata_comp"]],
        collapse_stanvars(stanvars, "tdata", "end"),
     "}\n"
   )
   
   # generate parameters block
   scode_parameters <- paste0(
-    scode_predictor$par,
-    scode_ranef$par,
-    scode_Xme$par
+    scode_predictor[["par"]],
+    scode_ranef[["par"]],
+    scode_Xme[["par"]]
   )
   # prepare additional sampling from priors
   scode_rngprior <- stan_rngprior(
     prior = scode_prior,
     par_declars = scode_parameters,
-    gen_quantities = scode_predictor$gen_def,
+    gen_quantities = scode_predictor[["gen_def"]],
     prior_special = attr(prior, "special"),
     sample_prior = get_sample_prior(prior)
   )
   scode_parameters <- paste0(
     "parameters {\n",
       scode_parameters,
-      scode_rngprior$par,
+      scode_rngprior[["par"]],
       collapse_stanvars(stanvars, "parameters"),
     "}\n"
   )
@@ -218,29 +238,31 @@ make_stancode <- function(formula, data, family = gaussian(),
   # generate transformed parameters block
   scode_transformed_parameters <- paste0(
     "transformed parameters {\n",
-      scode_predictor$tpar_def,
-      scode_ranef$tpar_def,
-      scode_Xme$tpar_def,
+      scode_predictor[["tpar_def"]],
+      scode_ranef[["tpar_def"]],
+      scode_Xme[["tpar_def"]],
       collapse_stanvars(stanvars, "tparameters", "start"),
-      scode_predictor$tpar_prior,
-      scode_ranef$tpar_prior,
-      scode_Xme$tpar_prior,
-      scode_predictor$tpar_comp,
-      scode_ranef$tpar_comp,
-      scode_Xme$tpar_comp,
+      scode_predictor[["tpar_prior"]],
+      scode_ranef[["tpar_prior"]],
+      scode_Xme[["tpar_prior"]],
+      scode_predictor[["tpar_comp"]],
+      scode_predictor[["tpar_reg_prior"]],
+      scode_ranef[["tpar_comp"]],
+      scode_Xme[["tpar_comp"]],
       collapse_stanvars(stanvars, "tparameters", "end"),
     "}\n"
   )
   
   # combine likelihood with prior part
+  not_const <- str_if(!normalize, " not")
   scode_model <- paste0(
     "model {\n",
       collapse_stanvars(stanvars, "model", "start"),
-      "  // likelihood including all constants\n",
+      "  // likelihood", not_const, " including constants\n",
       "  if (!prior_only) {\n",
-      scode_predictor$model_lik,
+      scode_predictor[["model_lik"]],
       "  }\n", 
-      "  // priors including all constants\n", 
+      "  // priors", not_const, " including constants\n",
       scode_prior, 
       collapse_stanvars(stanvars, "model", "end"),
     "}\n"
@@ -248,15 +270,15 @@ make_stancode <- function(formula, data, family = gaussian(),
   # generate generated quantities block
   scode_generated_quantities <- paste0(
     "generated quantities {\n",
-      scode_predictor$gen_def,
-      scode_ranef$gen_def,
-      scode_Xme$gen_def,
-      scode_rngprior$gen_def,
+      scode_predictor[["gen_def"]],
+      scode_ranef[["gen_def"]],
+      scode_Xme[["gen_def"]],
+      scode_rngprior[["gen_def"]],
       collapse_stanvars(stanvars, "genquant", "start"),
-      scode_predictor$gen_comp,
-      scode_ranef$gen_comp,
-      scode_rngprior$gen_comp,
-      scode_Xme$gen_comp,
+      scode_predictor[["gen_comp"]],
+      scode_ranef[["gen_comp"]],
+      scode_rngprior[["gen_comp"]],
+      scode_Xme[["gen_comp"]],
       collapse_stanvars(stanvars, "genquant", "end"),
     "}\n"
   )
@@ -298,13 +320,51 @@ print.brmsmodel <- function(x, ...) {
 #' @param version Logical; indicates if the first line containing
 #'   the \pkg{brms} version number should be included.
 #'   Defaults to \code{TRUE}.
-#' @param ... Currently ignored.
+#' @param regenerate Logical; indicates if the Stan code should
+#'   be regenerated with the current \pkg{brms} version.
+#'   By default, \code{regenerate} will be \code{FALSE} unless required
+#'   to be \code{TRUE} by other arguments.
+#' @param threads Controls whether the Stan code should be threaded.
+#'   See \code{\link{threading}} for details.
+#' @param ... Further arguments passed to \code{\link{make_stancode}} if the
+#'   Stan code is regenerated.
 #' 
 #' @return Stan model code for further processing.
 #' 
 #' @export
-stancode.brmsfit <- function(object, version = TRUE, ...) {
-  out <- object$model
+stancode.brmsfit <- function(object, version = TRUE, regenerate = NULL, 
+                             threads = NULL, ...) {
+  if (is.null(regenerate)) {
+    # determine whether regenerating the Stan code is required
+    regenerate <- FALSE
+    cl <- match.call()
+    if ("threads" %in% names(cl)) {
+      threads <- validate_threads(threads)
+      if (use_threading(threads) && !use_threading(object$threads) ||
+          !use_threading(threads) && use_threading(object$threads)) {
+        # threading changed; regenerated Stan code
+        regenerate <- TRUE
+      }
+      object$threads <- threads
+    }
+  }
+  regenerate <- as_one_logical(regenerate)
+  if (regenerate) {
+    object <- restructure(object)
+    out <- make_stancode(
+      formula = object$formula,
+      data = object$data,
+      prior = object$prior,
+      data2 = object$data2,
+      stanvars = object$stanvars,
+      sample_prior = get_sample_prior(object$prior),
+      threads = object$threads,
+      ...
+    )
+  } else {
+    # extract Stan code unaltered
+    out <- object$model
+  }
   if (!version) {
     out <- sub("^[^\n]+[[:digit:]]\\.[^\n]+\n", "", out) 
   }
@@ -336,3 +396,25 @@ expand_include_statements <- function(model) {
   }
   model
 }
+
+# check if Stan code includes normalization constants
+is_normalized <- function(stancode) {
+  !grepl("_lup(d|m)f\\(", stancode)
+}
+
+# Normalizes Stan code to avoid triggering refit after whitespace and
+# comment changes in the generated code.
+# In some distant future, StanC3 may provide its own normalizing functions,
+# until then this is a set of regex hacks.
+# @param x a string containing the Stan code
+normalize_stancode <- function(x) {
+  x <- as_one_character(x)
+  # Remove single-line comments
+  x <- gsub("//[^\n\r]*[\n\r]", " ", x)
+  x <- gsub("//[^\n\r]*$", " ", x)
+  # Remove multi-line comments
+  x <- gsub("/\\*([^*]*(\\*[^/])?)*\\*/", " ", x)
+  # Standardize whitespace (including newlines)
+  x <- gsub("[[:space:]]+"," ", x)
+  trimws(x)
+}
