@@ -33,17 +33,20 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
     autocor = autocor, cov_ranef = cov_ranef
   )
   bterms <- brmsterms(formula)
-  data <- validate_data(data, bterms = bterms, knots = knots)
-  prior <- validate_prior(
-    prior, bterms = bterms, data = data,
-    sample_prior = sample_prior,
-    require_nlpar_prior = FALSE
-  )
   data2 <- validate_data2(
     data2, bterms = bterms, 
     get_data2_autocor(formula),
     get_data2_cov_ranef(formula)
   )
+  data <- validate_data(
+    data, bterms = bterms, 
+    knots = knots, data2 = data2
+  )
+  prior <- .validate_prior(
+    prior, bterms = bterms, data = data,
+    sample_prior = sample_prior,
+    require_nlpar_prior = FALSE
+  )
   stanvars <- validate_stanvars(stanvars)
   threads <- validate_threads(threads)
   .make_standata(
@@ -77,9 +80,10 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
   if (!only_response) {
     ranef <- tidy_ranef(bterms, data, old_levels = basis$levels)
     meef <- tidy_meef(bterms, data, old_levels = basis$levels)
+    index <- tidy_index(bterms, data)
     c(out) <- data_predictor(
       bterms, data = data, prior = prior, data2 = data2,
-      ranef = ranef, basis = basis
+      ranef = ranef, index = index, basis = basis
     )
     c(out) <- data_gr_global(ranef, data2 = data2)
     c(out) <- data_Xme(meef, data = data)
@@ -105,11 +109,11 @@ make_standata <- function(formula, data, family = gaussian(), prior = NULL,
     # allows to recover the original order of the data
     attr(out, "old_order") <- attr(data, "old_order")
     # ensures current grouping levels are known in post-processing
-    attr(out, "levels") <- get_levels(
-      tidy_meef(bterms, data), tidy_ranef(bterms, data)
-    )
+    ranef_new <- tidy_ranef(bterms, data)
+    meef_new <- tidy_meef(bterms, data)
+    attr(out, "levels") <- get_levels(ranef_new, meef_new)
   }
-  structure(out, class = "standata")
+  structure(out, class = c("standata", "list"))
 }
 
 #' Extract data passed to Stan
@@ -137,18 +141,16 @@ standata.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
   on.exit(options(.brmsfit_version = NULL))
   
   object <- exclude_terms(object, incl_autocor = incl_autocor)
-  newdata2 <- use_alias(newdata2, new_objects)
   formula <- update_re_terms(object$formula, re_formula)
   bterms <- brmsterms(formula)
-  data <- current_data(object, newdata, re_formula = re_formula, ...)
-  stanvars <- object$stanvars
-  threads <- object$threads
-  if (is.null(newdata2)) {
-    data2 <- object$data2
-  } else {
-    data2 <- validate_data2(newdata2, bterms = bterms)
-    stanvars <- add_newdata_stanvars(stanvars, data2)
-  }
+  
+  newdata2 <- use_alias(newdata2, new_objects)
+  data2 <- current_data2(object, newdata2)
+  data <- current_data(
+    object, newdata, newdata2 = data2, 
+    re_formula = re_formula, ...
+  )
+  stanvars <- add_newdata_stanvars(object$stanvars, data2)
   
   basis <- NULL
   if (!is.null(newdata)) {
@@ -159,7 +161,7 @@ standata.brmsfit <- function(object, newdata = NULL, re_formula = NULL,
   .make_standata(
     bterms, data = data, prior = object$prior,
     data2 = data2, stanvars = stanvars, 
-    threads = threads, basis = basis, ...
+    threads = object$threads, basis = basis, ...
   )
 }
 
@@ -238,7 +240,7 @@ standata_basis_sm <- function(x, data, ...) {
   if (length(smterms)) {
     knots <- get_knots(data)
     data <- rm_attr(data, "terms")
-    # the spline penality has changed in 2.8.7 (#646)
+    # the spline penalty has changed in 2.8.7 (#646)
     diagonal.penalty <- !require_old_default("2.8.7")
     gam_args <- list(
       data = data, knots = knots, 
