@@ -1,15 +1,36 @@
-# rename parameters (and possibly change their dimensions) 
-# within the stanfit object to ensure reasonable parameter names
-# @param x a brmsfit obejct
-# @return a brmfit object with adjusted parameter names and dimensions
+#' Rename Parameters
+#' 
+#' Rename parameters within the \code{stanfit} object after model fitting to
+#' ensure reasonable parameter names. This function is usually called
+#' automatically by \code{\link{brm}} and users will rarely be required to call
+#' it themselves.
+#' 
+#' @param x A brmsfit object.
+#' @return A brmfit object with adjusted parameter names.
+#' 
+#' @examples
+#' \dontrun{
+#' # fit a model manually via rstan
+#' scode <- make_stancode(count ~ Trt, data = epilepsy)
+#' sdata <- make_standata(count ~ Trt, data = epilepsy)
+#' stanfit <- rstan::stan(model_code = scode, data = sdata)
+#' 
+#' # feed the Stan model back into brms
+#' fit <- brm(count ~ Trt, data = epilepsy, empty = TRUE)
+#' fit$fit <- stanfit
+#' fit <- rename_pars(fit)
+#' summary(fit)
+#' }
+#' 
+#' @export
 rename_pars <- function(x) {
   if (!length(x$fit@sim)) {
     return(x) 
   }
-  bterms <- parse_bf(x$formula)
+  bterms <- brmsterms(x$formula)
   data <- model.frame(x)
   meef <- tidy_meef(bterms, data)
-  pars <- parnames(x)
+  pars <- variables(x)
   # find positions of parameters and define new names
   change <- c(
     change_effects(bterms, data = data, pars = pars),
@@ -19,6 +40,7 @@ rename_pars <- function(x) {
   # perform the actual renaming in x$fit@sim
   x <- save_old_par_order(x)
   x <- do_renaming(x, change)
+  x$fit <- repair_stanfit_names(x$fit)
   x <- compute_quantities(x)
   x <- reorder_pars(x)
   x
@@ -91,6 +113,7 @@ change_fe <- function(bterms, data, pars) {
     bnames <- paste0(b, "_", fixef)
     lc(out) <- clist(pos, bnames)
     c(out) <- change_prior(b, pars, names = fixef)
+    c(out) <- change_special_prior_local(bterms, fixef, pars)
   }
   out
 }
@@ -380,6 +403,21 @@ change_re_levels <- function(ranef, pars)  {
   out
 }
 
+# rename parameters related to special priors
+change_special_prior_local <- function(bterms, coef, pars) {
+  out <- list()
+  p <- combine_prefix(bterms)
+  # rename parameters related to the R2D2 prior
+  pos_R2D2_phi <- grepl(paste0("^R2D2_phi", p), pars)
+  if (any(pos_R2D2_phi)) {
+    phi <- paste0("R2D2_phi", p)
+    new_phi <- paste0(phi, "_", coef)
+    lc(out) <- clist(pos_R2D2_phi, new_phi)
+    c(out) <- change_prior(phi, pars, names = coef, is_vector = TRUE)
+  }
+  out
+}
+
 # helps in renaming prior parameters
 # @param class the class of the parameters
 # @param pars names of all parameters in the model
@@ -398,6 +436,12 @@ change_prior <- function(class, pars, names = NULL, new_class = class,
       pars[pos_priors]
     )
     if (is_vector) {
+      if (!is.null(names)) {
+        .names <- paste0("_", names)
+        for (i in seq_along(priors)) {
+          priors[i] <- gsub("\\[[[:digit:]]+\\]$", .names[i], priors[i])
+        }
+      }
       lc(out) <- clist(pos_priors, priors)
     } else {
       digits <- sapply(priors, function(prior) {
@@ -480,7 +524,7 @@ do_renaming <- function(x, change) {
   x
 }
 
-# order parameter samples after parameter class
+# order parameter draws after parameter class
 # @param x brmsfit object
 reorder_pars <- function(x) {
   all_classes <- unique(c(
@@ -488,10 +532,11 @@ reorder_pars <- function(x) {
     "car", "sdcar", "cosy", "sd", "cor", "df", "sds", "sdgp", 
     "lscale", valid_dpars(x), "Intercept", "tmp", "rescor", 
     "delta", "lasso", "simo", "r", "s", "zgp", "rcar", "sbhaz", 
-    "Ymi", "Yl", "meanme", "sdme", "corme", "Xme", "prior", "lp"
+    "R2D2", "Ymi", "Yl", "meanme", "sdme", "corme", "Xme", "prior", 
+    "lp"
   ))
   # reorder parameter classes
-  class <- get_matches("^[^[:digit:]_]+", x$fit@sim$pars_oi)
+  class <- get_matches("^[^_]+", x$fit@sim$pars_oi)
   new_order <- order(
     factor(class, levels = all_classes),
     !grepl("_Intercept(_[[:digit:]]+)?$", x$fit@sim$pars_oi)
@@ -537,7 +582,7 @@ compute_xi <- function(x, ...) {
 
 #' @export
 compute_xi.brmsfit <- function(x, ...) {
-  if (!any(grepl("^tmp_xi(_|$)", parnames(x)))) {
+  if (!any(grepl("^tmp_xi(_|$)", variables(x)))) {
     return(x)
   }
   draws <- try(extract_draws(x))
@@ -550,7 +595,7 @@ compute_xi.brmsfit <- function(x, ...) {
 }
 
 #' @export
-compute_xi.mvbrmsdraws <- function(x, fit, ...) {
+compute_xi.mvbrmsprep <- function(x, fit, ...) {
   stopifnot(is.brmsfit(fit))
   for (resp in names(x$resps)) {
     fit <- compute_xi(x$resps[[resp]], fit = fit, ...)
@@ -559,11 +604,11 @@ compute_xi.mvbrmsdraws <- function(x, fit, ...) {
 }
 
 #' @export
-compute_xi.brmsdraws <- function(x, fit, ...) {
+compute_xi.brmsprep <- function(x, fit, ...) {
   stopifnot(is.brmsfit(fit))
   resp <- usc(x$resp)
   tmp_xi_name <- paste0("tmp_xi", resp)
-  if (!tmp_xi_name %in% parnames(fit)) {
+  if (!tmp_xi_name %in% variables(fit)) {
     return(fit)
   }
   mu <- get_dpar(x, "mu")
@@ -578,7 +623,7 @@ compute_xi.brmsdraws <- function(x, fit, ...) {
   samp_chain <- length(xi) / fit$fit@sim$chains
   for (i in seq_len(fit$fit@sim$chains)) {
     xi_part <- xi[((i - 1) * samp_chain + 1):(i * samp_chain)]
-    # add warmup samples not used anyway
+    # add warmup draws not used anyway
     xi_part <- c(rep(0, fit$fit@sim$warmup2[1]), xi_part)
     fit$fit@sim$samples[[i]][[xi_name]] <- xi_part
   }
