@@ -26,7 +26,8 @@
 stan_prior <- function(prior, class, coef = NULL, group = NULL, 
                        type = "real", dim = "", coef_type = "real",
                        prefix = "", suffix = "", broadcast = "vector", 
-                       comment = "", px = list()) {
+                       header_type = "", comment = "", px = list(),
+                       normalize = TRUE) {
   prior_only <- isTRUE(attr(prior, "sample_prior") == "only")
   prior <- subset2(
     prior, class = class, coef = c(coef, ""), 
@@ -54,9 +55,10 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
     if (length(unique(base_prior)) > 1L) {
       # define prior for single coefficients manually
       # as there is not single base_prior anymore
-      prior_of_coefs <- prior[nzchar(prior$coef), vars_prefix()]
-      take <- match_rows(prior_of_coefs, upx)
-      prior[nzchar(prior$coef), "prior"] <- base_prior[take]
+      take_coef_prior <- nzchar(prior$coef) & !nzchar(prior$prior)
+      prior_of_coefs <- prior[take_coef_prior, vars_prefix()]
+      take_base_prior <- match_rows(prior_of_coefs, upx)
+      prior$prior[take_coef_prior] <- base_prior[take_base_prior]
     }
     base_prior <- base_prior[1]
     bound <- ""
@@ -98,26 +100,38 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
           used_base_prior <- TRUE
           coef_prior <- base_prior
         }
+        if (!stan_is_constant_prior(coef_prior)) {
+          # all parameters with non-constant priors are estimated 
+          c(estimated_coef_indices) <- list(index)
+        }
         if (nzchar(coef_prior)) {
           # implies a proper prior or constant
-          par_ij <- paste0(par, collapse("[", index, "]"))
+          if (type == coef_type && !nzchar(dim)) {
+            # the single coefficient of that parameter equals the parameter
+            stopifnot(all(index == 1L))
+            par_ij <- par
+          } else {
+            par_ij <- paste0(par, collapse("[", index, "]")) 
+          }
           if (stan_is_constant_prior(coef_prior)) {
             coef_prior <- stan_constant_prior(
               coef_prior, par_ij, broadcast = broadcast
             )
             str_add(out$tpar_prior) <- paste0(coef_prior, ";\n")
           } else {
-            c(estimated_coef_indices) <- list(index)
             coef_prior <- stan_target_prior(
               coef_prior, par_ij, broadcast = broadcast, 
-              bound = bound, resp = px$resp[1]
+              bound = bound, resp = px$resp[1], normalize = normalize
             )
             str_add(out$prior) <- paste0(tp(), coef_prior, ";\n") 
           }
         }
       }
     }
-    has_estimated_priors <- isTRUE(nzchar(out$prior))
+    # the base prior may be improper flat in which no Stan code is added
+    # but we still have estimated coefficients if the base prior is used
+    has_estimated_priors <- isTRUE(nzchar(out$prior)) ||
+      used_base_prior && !stan_is_constant_prior(base_prior)
     has_constant_priors <- isTRUE(nzchar(out$tpar_prior))
     if (has_estimated_priors && has_constant_priors) {
       # need to mix definition in the parameters and transformed parameters block
@@ -148,7 +162,7 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
     } else {
       target_base_prior <- stan_target_prior(
         base_prior, par = par, ncoef = ncoef, bound = bound,
-        broadcast = broadcast, resp = px$resp[1]
+        broadcast = broadcast, resp = px$resp[1], normalize = normalize
       )
       str_add(out$prior) <- paste0(tp(), target_base_prior, ";\n")
     }
@@ -165,6 +179,9 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
       # parameter can be defined in the parameters block
       str_add(out$par) <- par_definition
     }
+    if (nzchar(header_type)) {
+      str_add(out$pll_args) <- glue(", {header_type} {par}") 
+    }
   } else {
     if (has_constant_priors) {
       stop2("Cannot fix parameter '", par, "' in this model.")
@@ -174,7 +191,7 @@ stan_prior <- function(prior, class, coef = NULL, group = NULL,
   if (prior_only && has_improper_prior) {
     stop2("Sampling from priors is not possible as ", 
           "some parameters have no proper priors. ",
-          "Error occured for parameter '", par, "'.")
+          "Error occurred for parameter '", par, "'.")
   }
   out
 }
@@ -210,7 +227,7 @@ stan_base_prior <- function(prior) {
 # @param name of the response variable
 # @return a character string defining the prior in Stan language
 stan_target_prior <- function(prior, par, ncoef = 0, broadcast = "vector",
-                              bound = "", resp = "") {
+                              bound = "", resp = "", normalize = TRUE) {
   prior <- gsub("[[:space:]]+\\(", "(", prior)
   prior_name <- get_matches(
     "^[^\\(]+(?=\\()", prior, perl = TRUE, simplify = FALSE
@@ -224,36 +241,44 @@ stan_target_prior <- function(prior, par, ncoef = 0, broadcast = "vector",
   prior_args <- rep(NA, length(prior))
   for (i in seq_along(prior)) {
     prior_args[i] <- sub(glue("^{prior_name[i]}\\("), "", prior[i])
+    prior_args[i] <- sub(")$", "", prior_args[i])
   }
   if (broadcast == "matrix" && ncoef > 0) {
     # apply a scalar prior to all elements of a matrix 
     par <- glue("to_vector({par})")
   }
   
-  out <- glue("{prior_name}_lpdf({par} | {prior_args}")
+  if (nzchar(prior_args)) {
+    str_add(prior_args, start = TRUE) <- " | "
+  }
+  lpdf <- stan_lpdf_name(normalize)
+  out <- glue("{prior_name}_{lpdf}({par}{prior_args})")
   par_class <- unique(get_matches("^[^_]+", par))
   par_bound <- par_bounds(par_class, bound, resp = resp)
   prior_bound <- prior_bounds(prior_name)
   trunc_lb <- is.character(par_bound$lb) || par_bound$lb > prior_bound$lb
   trunc_ub <- is.character(par_bound$ub) || par_bound$ub < prior_bound$ub
-  if (trunc_lb || trunc_ub) {
-    wsp <- wsp(nsp = 4)
-    # scalar parameters are of length 1 but have no coefficients
-    ncoef <- max(1, ncoef)
-    if (trunc_lb && !trunc_ub) {
-      str_add(out) <- glue(
-        "\n{wsp}- {ncoef} * {prior_name}_lccdf({par_bound$lb} | {prior_args}"
-      )
-    } else if (!trunc_lb && trunc_ub) {
-      str_add(out) <- glue(
-        "\n{wsp}- {ncoef} * {prior_name}_lcdf({par_bound$ub} | {prior_args}"
-      )
-    } else if (trunc_lb && trunc_ub) {
-      str_add(out) <- glue(
-        "\n{wsp}- {ncoef} * log_diff_exp(", 
-        "{prior_name}_lcdf({par_bound$ub} | {prior_args}, ",
-        "{prior_name}_lcdf({par_bound$lb} | {prior_args})"
-      )
+  if (normalize) {
+    # obtain correct normalization constants for truncated priors
+    if (trunc_lb || trunc_ub) {
+      wsp <- wsp(nsp = 4)
+      # scalar parameters are of length 1 but have no coefficients
+      ncoef <- max(1, ncoef)
+      if (trunc_lb && !trunc_ub) {
+        str_add(out) <- glue(
+          "\n{wsp}- {ncoef} * {prior_name}_lccdf({par_bound$lb}{prior_args})"
+        )
+      } else if (!trunc_lb && trunc_ub) {
+        str_add(out) <- glue(
+          "\n{wsp}- {ncoef} * {prior_name}_lcdf({par_bound$ub}{prior_args})"
+        )
+      } else if (trunc_lb && trunc_ub) {
+        str_add(out) <- glue(
+          "\n{wsp}- {ncoef} * log_diff_exp(", 
+          "{prior_name}_lcdf({par_bound$ub}{prior_args}), ",
+          "{prior_name}_lcdf({par_bound$lb}{prior_args}))"
+        )
+      }
     }
   }
   out
@@ -288,14 +313,14 @@ stan_constant_prior <- function(prior, par, ncoef = 0, broadcast = "vector") {
 
 # Stan code for global parameters of special priors
 # currently implemented are horseshoe and lasso
-stan_special_prior_global <- function(bterms, data, prior, ...) {
+stan_special_prior_global <- function(bterms, data, prior, normalize, ...) {
   out <- list()
   tp <- tp()
+  lpdf <- stan_lpdf_name(normalize)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
-  prefix <- combine_prefix(px, keep_mu = TRUE)
-  special <- attributes(prior)$special[[prefix]]
-  if (!is.null(special[["hs_df"]])) {
+  special <- get_special_prior(prior, px)
+  if (!is.null(special$horseshoe)) {
     str_add(out$data) <- glue(
       "  // data for the horseshoe prior\n",
       "  real<lower=0> hs_df{p};  // local degrees of freedom\n",
@@ -306,21 +331,44 @@ stan_special_prior_global <- function(bterms, data, prior, ...) {
     )
     str_add(out$par) <- glue(
       "  // horseshoe shrinkage parameters\n",
-      "  real<lower=0> hs_global{p}[2];  // global shrinkage parameters\n",
-      "  real<lower=0> hs_c2{p};  // slab regularization parameter\n"
+      "  real<lower=0> hs_global{p};  // global shrinkage parameters\n",
+      "  real<lower=0> hs_slab{p};  // slab regularization parameter\n"
     )
-    global_args <- glue("0.5 * hs_df_global{p}")
-    global_args <- sargs(global_args, global_args)
-    c2_args <- glue("0.5 * hs_df_slab{p}")
-    c2_args <- sargs(c2_args, c2_args)
+    hs_scale_global <- glue("hs_scale_global{p}")
+    if (isTRUE(special$horseshoe$autoscale)) {
+      str_add(hs_scale_global) <- glue(" * sigma{usc(px$resp)}")
+    }
     str_add(out$prior) <- glue(
-      "{tp}normal_lpdf(hs_global{p}[1] | 0, 1)\n",
-      "    - 1 * log(0.5);\n",
-      "{tp}inv_gamma_lpdf(hs_global{p}[2] | {global_args});\n",
-      "{tp}inv_gamma_lpdf(hs_c2{p} | {c2_args});\n"
+      "{tp}student_t_{lpdf}(hs_global{p} | hs_df_global{p}, 0, {hs_scale_global})",
+      str_if(normalize, "\n    - 1 * log(0.5)"), ";\n",
+      "{tp}inv_gamma_{lpdf}(hs_slab{p} | 0.5 * hs_df_slab{p}, 0.5 * hs_df_slab{p});\n"
     )
   }
-  if (!is.null(special[["lasso_df"]])) {
+  if (!is.null(special$R2D2)) {
+    str_add(out$data) <- glue(
+      "  // data for the R2D2 prior\n",
+      "  real<lower=0> R2D2_mean_R2{p};  // mean of the R2 prior\n",
+      "  real<lower=0> R2D2_prec_R2{p};  // precision of the R2 prior\n"
+    )
+    str_add(out$par) <- glue(
+      "  // R2D2 shrinkage parameters\n",
+      "  real<lower=0,upper=1> R2D2_R2{p};  // R2 parameter\n"
+    )
+    if (isTRUE(special$R2D2$autoscale)) {
+      var_mult <- glue("sigma{usc(px$resp)}^2 * ")
+    }
+    str_add(out$tpar_def) <- glue(
+      "  real R2D2_tau2{p};  // global R2D2 scale parameter\n"
+    )
+    str_add(out$tpar_comp) <- glue(
+      "  R2D2_tau2{p} = {var_mult}R2D2_R2{p} / (1 - R2D2_R2{p});\n"
+    )
+    str_add(out$prior) <- glue(
+      "{tp}beta_{lpdf}(R2D2_R2{p} | R2D2_mean_R2{p} * R2D2_prec_R2{p}, ",
+      "(1 - R2D2_mean_R2{p}) * R2D2_prec_R2{p});\n"
+    )
+  }
+  if (!is.null(special$lasso)) {
     str_add(out$data) <- glue(
       "  // data for the lasso prior\n",
       "  real<lower=0> lasso_df{p};  // prior degrees of freedom\n",
@@ -331,7 +379,7 @@ stan_special_prior_global <- function(bterms, data, prior, ...) {
       "  real<lower=0> lasso_inv_lambda{p};\n"
     )
     str_add(out$prior) <- glue(
-      "{tp}chi_square_lpdf(lasso_inv_lambda{p} | lasso_df{p});\n"
+      "{tp}chi_square_{lpdf}(lasso_inv_lambda{p} | lasso_df{p});\n"
     )
   }
   out
@@ -346,41 +394,61 @@ stan_special_prior_global <- function(bterms, data, prior, ...) {
 # @param center_X is the design matrix centered?
 # @param suffix optional suffix of the 'b' coefficient vector
 stan_special_prior_local <- function(prior, class, ncoef, px, 
-                                     center_X = FALSE, suffix = "") {
+                                     center_X = FALSE, suffix = "",
+                                     normalize = TRUE) {
   class <- as_one_character(class)
   stopifnot(class %in% c("b", "bsp"))
   out <- list()
+  lpdf <- stan_lpdf_name(normalize)
   p <- usc(combine_prefix(px))
   sp <- paste0(sub("^b", "", class), p)
   ct <- str_if(center_X, "c")
   tp <- tp()
-  prefix <- combine_prefix(px, keep_mu = TRUE)
-  special <- attributes(prior)$special[[prefix]]
-  if (!is.null(special[["hs_df"]])) {
+  special <- get_special_prior(prior, px)
+  if (!is.null(special$horseshoe)) {
     str_add(out$par) <- glue(
       "  // local parameters for horseshoe prior\n",
       "  vector[K{ct}{sp}] zb{sp};\n",
-      "  vector<lower=0>[K{ct}{sp}] hs_local{sp}[2];\n"
+      "  vector<lower=0>[K{ct}{sp}] hs_local{sp};\n"
     )
-    hs_scale_global <- glue("hs_scale_global{p}")
-    if (isTRUE(special[["hs_autoscale"]])) {
-      str_add(hs_scale_global) <- glue(" * sigma{usc(px$resp)}")
-    }
     hs_args <- sargs(
       glue("zb{sp}"), glue("hs_local{sp}"), glue("hs_global{p}"), 
-      hs_scale_global, glue("hs_scale_slab{p}^2 * hs_c2{p}")
+      glue("hs_scale_slab{p}^2 * hs_slab{p}")
     )
-    str_add(out$tpar_prior) <- glue(
+    str_add(out$tpar_reg_prior) <- glue(
       "  // compute actual regression coefficients\n",
-      "  b{sp}{suffix} = horseshoe({hs_args});\n"
+      "  b{suffix}{sp} = horseshoe({hs_args});\n"
     )
-    local_args <- glue("0.5 * hs_df{p}")
-    local_args <- sargs(local_args, local_args)
     str_add(out$prior) <- glue(
-      "{tp}normal_lpdf(zb{sp} | 0, 1);\n",
-      "{tp}normal_lpdf(hs_local{sp}[1] | 0, 1)\n", 
-      "    - {ncoef} * log(0.5);\n",
-      "{tp}inv_gamma_lpdf(hs_local{sp}[2] | {local_args});\n"
+      "{tp}std_normal_{lpdf}(zb{sp});\n",
+      "{tp}student_t_{lpdf}(hs_local{sp} | hs_df{p}, 0, 1)",
+      str_if(normalize, "\n    - rows(hs_local{sp}) * log(0.5)"), ";\n"
+    )
+  }
+  if (!is.null(special$R2D2)) {
+    if (class != "b") {
+      stop2("The R2D2 prior does not yet support special coefficient classes.")
+    }
+    m1 <- str_if(center_X, " -1")
+    str_add(out$data) <- glue(
+      "  // concentration vector of the D2 prior\n",
+      "  vector<lower=0>[K{sp}{m1}] R2D2_cons_D2{sp};\n"
+    )
+    str_add(out$par) <- glue(
+      "  // local parameters for the R2D2 prior\n",
+      "  vector[K{ct}{sp}] zb{sp};\n",
+      "  simplex[K{ct}{sp}] R2D2_phi{sp};\n"
+    )
+    R2D2_args <- sargs(
+      glue("zb{sp}"), glue("R2D2_phi{sp}"), glue("R2D2_tau2{p}")
+    )
+    str_add(out$tpar_reg_prior) <- glue(
+      "  // compute actual regression coefficients\n",
+      "  b{suffix}{sp} = R2D2({R2D2_args});\n"
+    )
+    str_add(out$prior) <- glue(
+      "{tp}std_normal_{lpdf}(zb{sp});\n",
+      "{tp}dirichlet_{lpdf}(R2D2_phi{sp} | R2D2_cons_D2{p});\n"
     )
   }
   out
@@ -395,20 +463,20 @@ stan_unchecked_prior <- function(prior) {
     return("")
   }
   prior <- subset2(prior, class = "")
-  collapse("  ", prior$prior, ";\n") 
+  collapse("  ", prior$prior, ";\n")
 }
 
 # Stan code to sample separately from priors
-# @param sample_prior take samples from priors?
 # @param prior character string taken from stan_prior
 # @param par_declars the parameters block of the Stan code
 #     required to extract boundaries
 # @param gen_quantities Stan code from the generated quantities block
 # @param prior_special a list of values pertaining to special priors
 #   such as horseshoe or lasso
-stan_rngprior <- function(sample_prior, prior, par_declars,
-                          gen_quantities, prior_special) {
-  if (!sample_prior %in% "yes") {
+# @param sample_prior take draws from priors?
+stan_rngprior <- function(prior, par_declars, gen_quantities, 
+                          prior_special, sample_prior = "yes") {
+  if (!is_equal(sample_prior, "yes")) {
     return(list())
   }
   prior <- strsplit(gsub(" |\\n", "", prior), ";")[[1]]
@@ -416,6 +484,10 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
   D <- data.frame(prior = prior[nzchar(prior)])
   pars_regex <- "(?<=(_lpdf\\())[^|]+" 
   D$par <- get_matches(pars_regex, D$prior, perl = TRUE, first = TRUE)
+  # 'std_normal' has no '|' and thus the above regex matches too much
+  np <- !grepl("\\|", D$prior)
+  np_regex <- ".+(?=\\)$)"
+  D$par[np] <- get_matches(np_regex, D$par[np], perl = TRUE, first = TRUE)
   # 'to_vector' should be removed from the parameter names
   has_tv <- grepl("^to_vector\\(", D$par)
   tv_regex <- "(^to_vector\\()|(\\)(?=((\\[[[:digit:]]+\\])?)$))"
@@ -446,6 +518,10 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
   D$dist <- sub("corr_cholesky$", "corr", D$dist)
   args_regex <- "(?<=\\|)[^$\\|]+(?=\\)($|-))"
   D$args <- get_matches(args_regex, D$prior, perl = TRUE, first = TRUE)
+  # 'std_normal_rng' does not exist in Stan
+  has_std_normal <- D$dist == "std_normal"
+  D$dist[has_std_normal] <- "normal"
+  D$args[has_std_normal] <- "0,1"
   
   # extract information from the initial parameter definition
   par_declars <- unlist(strsplit(par_declars, "\n", fixed = TRUE))
@@ -467,7 +543,7 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
     k <- which(grepl(paste0("^", all_pars[i]), D$par))
     D$dim[k] <- all_dims[i]
     D$bounds[k] <- all_bounds[i]
-    if (grepl("^((simo_)|(theta))", all_pars[i])) {
+    if (grepl("^((simo_)|(theta)|(R2D2_phi))", all_pars[i])) {
       D$type[k] <- all_types[i]
     }
   }
@@ -485,7 +561,7 @@ stan_rngprior <- function(sample_prior, prior, par_declars,
   D$args <- paste0(ifelse(D$lkj, paste0(D$dim, ","), ""), D$args)
   D$lkj_index <- ifelse(D$lkj, "[1, 2]", "")
   D$prior_par <- glue("prior_{D$par}")
-  str_add(out$gen_def) <- "  // additionally draw samples from priors\n"
+  str_add(out$gen_def) <- "  // additionally sample draws from priors\n"
   str_add(out$gen_def) <- cglue(
     "  {D$type} {D$prior_par} = {D$dist}_rng({D$args}){D$lkj_index};\n"
   )
@@ -516,13 +592,6 @@ stan_is_constant_prior <- function(prior) {
   grepl("^constant\\(", prior)
 }
 
-# indicate if the horseshoe prior is used in the predictor term
-stan_use_horseshoe <- function(bterms, prior) {
-  prefix <- combine_prefix(bterms, keep_mu = TRUE)
-  special <- attr(prior, "special")[[prefix]]
-  !is.null(special[["hs_df"]])
-}
-
 # extract Stan boundaries expression from a string
 stan_extract_bounds <- function(x, bound = c("lower", "upper")) {
   bound <- match.arg(bound)
@@ -530,3 +599,13 @@ stan_extract_bounds <- function(x, bound = c("lower", "upper")) {
   regex <- glue("(?<={bound}=)[^,>]*")
   get_matches(regex, x, perl = TRUE, first = TRUE)
 }
+
+# choose the right suffix for Stan probability densities
+stan_lpdf_name <- function(normalize, int = FALSE) {
+  if (normalize) {
+    out <- ifelse(int, "lpmf", "lpdf")
+  } else {
+    out <- ifelse(int, "lupmf", "lupdf")
+  }
+  out
+}
