@@ -2,43 +2,54 @@
 # of Stan code snippets to be pasted together later on
 
 # Stan code for the response variables
-stan_response <- function(bterms, data) {
+stan_response <- function(bterms, data, normalize) {
   stopifnot(is.brmsterms(bterms))
+  lpdf <- stan_lpdf_name(normalize)
   family <- bterms$family
   rtype <- str_if(use_int(family), "int", "real")
   multicol <- has_multicol(family)
   px <- check_prefix(bterms)
   resp <- usc(combine_prefix(px))
-  out <- list()
-  str_add(out$data) <- glue(
-    "  int<lower=1> N{resp};  // number of observations\n"
-  )
-  
+  out <- list(resp_type = rtype)
+  if (nzchar(resp)) {
+    # global N is defined elsewhere
+    str_add(out$data) <- glue(
+      "  int<lower=1> N{resp};  // number of observations\n"
+    )
+    str_add(out$pll_def) <- glue(
+      "  int N{resp} = end - start + 1;\n"
+    )
+  }
   if (has_cat(family)) {
     str_add(out$data) <- glue(
       "  int<lower=2> ncat{resp};  // number of categories\n"
     )
+    str_add(out$pll_args) <- glue(", data int ncat{resp}")
   }
   if (has_multicol(family)) {
     if (rtype == "real") {
       str_add(out$data) <- glue(
         "  vector[ncat{resp}] Y{resp}[N{resp}];  // response array\n"
       )
+      str_add(out$pll_args) <- glue(", data vector[] Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
         "  int Y{resp}[N{resp}, ncat{resp}];  // response array\n"
       )
+      str_add(out$pll_args) <- glue(", data int[,] Y{resp}")
     }
   } else {
     if (rtype == "real") {
-      # don't use real Y[n]
+      # type vector (instead of real[]) is required by some PDFs
       str_add(out$data) <- glue(
         "  vector[N{resp}] Y{resp};  // response variable\n"
       )
+      str_add(out$pll_args) <- glue(", data vector Y{resp}")
     } else if (rtype == "int") {
       str_add(out$data) <- glue(
         "  int Y{resp}[N{resp}];  // response variable\n"
       )
+      str_add(out$pll_args) <- glue(", data int[] Y{resp}")
     }
   }
   if (has_ndt(family)) {
@@ -50,11 +61,13 @@ stan_response <- function(bterms, data) {
     str_add(out$data) <- glue(
       "  int trials{resp}[N{resp}];  // number of trials\n"
     )
+    str_add(out$pll_args) <- glue(", data int[] trials{resp}")
   }
   if (is.formula(bterms$adforms$weights)) {
     str_add(out$data) <- glue(
       "  vector<lower=0>[N{resp}] weights{resp};  // model weights\n" 
     )
+    str_add(out$pll_args) <- glue(", data vector weights{resp}")
   }
   if (has_thres(family)) {
     groups <- get_thres_groups(family)
@@ -65,7 +78,7 @@ stan_response <- function(bterms, data) {
         "  int<lower=1> Jthres{resp}[N{resp}, 2];  // threshold indices\n"
       )
       str_add(out$tdata_def) <- glue(
-        "  int<lower=1> nmthres{resp} = prod(nthres{resp});", 
+        "  int<lower=1> nmthres{resp} = sum(nthres{resp});", 
         "  // total number of thresholds\n",
         "  int<lower=1> Kthres_start{resp}[ngrthres{resp}];", 
         "  // start index per threshold group\n",
@@ -80,10 +93,14 @@ stan_response <- function(bterms, data) {
         "    Kthres_end{resp}[i] = Kthres_end{resp}[i-1] + nthres{resp}[i];\n",
         "  }}\n"
       )
+      str_add(out$pll_args) <- glue(
+        ", data int[] nthres{resp}, data int[,] Jthres{resp}"
+      )
     } else {
       str_add(out$data) <- glue(
         "  int<lower=2> nthres{resp};  // number of thresholds\n"
       )
+      str_add(out$pll_args) <- glue(", data int nthres{resp}")
     }
   }
   if (is.formula(bterms$adforms$se)) {
@@ -93,11 +110,15 @@ stan_response <- function(bterms, data) {
     str_add(out$tdata_def) <- glue(
       "  vector<lower=0>[N{resp}] se2{resp} = square(se{resp});\n"
     )
+    str_add(out$pll_args) <- glue(
+      ", data vector se{resp}, data vector se2{resp}"
+    )
   }
   if (is.formula(bterms$adforms$dec)) {
     str_add(out$data) <- glue(
       "  int<lower=0,upper=1> dec{resp}[N{resp}];  // decisions\n"
     )
+    str_add(out$pll_args) <- glue(", data int[] dec{resp}")
   }
   if (is.formula(bterms$adforms$rate)) {
     str_add(out$data) <- glue(
@@ -108,21 +129,30 @@ stan_response <- function(bterms, data) {
       "  // log response denominator\n",
       "  vector[N{resp}] log_denom{resp} = log(denom{resp});\n"
     )
+    str_add(out$pll_args) <- glue(
+      ", data vector denom{resp}, data vector log_denom{resp}"
+    )
   }
   if (is.formula(bterms$adforms$cens)) {
     cens <- eval_rhs(bterms$adforms$cens)
     str_add(out$data) <- glue(
       "  int<lower=-1,upper=2> cens{resp}[N{resp}];  // indicates censoring\n"
     )
+    str_add(out$pll_args) <- glue(", data int[] cens{resp}")
     if (cens$vars$y2 != "NA") {
       # interval censoring is required
-      rcens <- str_if(rtype == "int", 
-                      glue("  int rcens{resp}[N{resp}];"), 
-                      glue("  vector[N{resp}] rcens{resp};")
-      )
-      str_add(out$data) <- glue(
-        rcens, "  // right censor points for interval censoring\n"
-      )
+      if (rtype == "int") {
+        str_add(out$data) <- glue(
+          "  int rcens{resp}[N{resp}];"
+        )
+        str_add(out$pll_args) <- glue(", data int[] rcens{resp}")
+      } else {
+        str_add(out$data) <- glue(
+          "  vector[N{resp}] rcens{resp};"
+        )
+        str_add(out$pll_args) <- glue(", data vector rcens{resp}")
+      }
+      str_add(out$data) <- "  // right censor points for interval censoring\n"
     }
   }
   bounds <- trunc_bounds(bterms, data = data)
@@ -130,33 +160,35 @@ stan_response <- function(bterms, data) {
     str_add(out$data) <- glue(
       "  {rtype} lb{resp}[N{resp}];  // lower truncation bounds;\n"
     )
+    str_add(out$pll_args) <- glue(", data {rtype}[] lb{resp}")
   }
   if (any(bounds$ub < Inf)) {
     str_add(out$data) <- glue(
       "  {rtype} ub{resp}[N{resp}];  // upper truncation bounds\n"
     )
+    str_add(out$pll_args) <- glue(", data {rtype}[] ub{resp}")
   }
   if (is.formula(bterms$adforms$mi)) {
+    # TODO: pass 'Ybounds' via 'standata' instead of hardcoding them
     Ybounds <- trunc_bounds(bterms, data, incl_family = TRUE, stan = TRUE)
     sdy <- get_sdy(bterms, data)
     if (is.null(sdy)) {
       # response is modeled without measurement error
-      str_add(out$par) <- glue(
-        "  vector{Ybounds}[Nmi{resp}] Ymi{resp};",
-        "  // estimated missings\n"
-      )
       str_add(out$data) <- glue(
         "  int<lower=0> Nmi{resp};  // number of missings\n",
-        "  int<lower=1> Jmi{resp}[Nmi{resp}];",  
-        "  // positions of missings\n"
+        "  int<lower=1> Jmi{resp}[Nmi{resp}];  // positions of missings\n"
       )
-      str_add(out$model_def) <- glue(
+      str_add(out$par) <- glue(
+        "  vector{Ybounds}[Nmi{resp}] Ymi{resp};  // estimated missings\n"
+      )
+      str_add(out$model_no_pll_def) <- glue(
         "  // vector combining observed and missing responses\n",
         "  vector[N{resp}] Yl{resp} = Y{resp};\n" 
       )
-      str_add(out$model_comp_basic) <- glue(
+      str_add(out$model_no_pll_comp_basic) <- glue(
         "  Yl{resp}[Jmi{resp}] = Ymi{resp};\n"
       )
+      str_add(out$pll_args) <- glue(", vector Yl{resp}")
     } else {
       str_add(out$data) <- glue(
         "  // data for measurement-error in the response\n",
@@ -169,10 +201,10 @@ stan_response <- function(bterms, data) {
         "  vector{Ybounds}[N{resp}] Yl{resp};  // latent variable\n"
       )
       str_add(out$prior) <- glue(
-        "  target += normal_lpdf(Y{resp}[Jme{resp}]",
-        " | Yl{resp}[Jme{resp}],", 
-        " noise{resp}[Jme{resp}]);\n"
+        "  target += normal_{lpdf}(Y{resp}[Jme{resp}]",
+        " | Yl{resp}[Jme{resp}], noise{resp}[Jme{resp}]);\n"
       )
+      str_add(out$pll_args) <- glue(", vector Yl{resp}")
     }
   }
   if (is.formula(bterms$adforms$vreal)) {
@@ -181,8 +213,9 @@ stan_response <- function(bterms, data) {
     k <- length(vreal$vars)
     str_add(out$data) <- cglue(
       "  // data for custom real vectors\n",
-      "  vector[N{resp}] vreal{seq_len(k)}{resp};\n"
+      "  real vreal{seq_len(k)}{resp}[N{resp}];\n"
     )
+    str_add(out$pll_args) <- cglue(", data real[] vreal{seq_len(k)}{resp}")
   }
   if (is.formula(bterms$adforms$vint)) {
     # vectors of integer values for use in custom families
@@ -192,6 +225,7 @@ stan_response <- function(bterms, data) {
       "  // data for custom integer vectors\n",
       "  int vint{seq_len(k)}{resp}[N{resp}];\n"
     )
+    str_add(out$pll_args) <- cglue(", data int[] vint{seq_len(k)}{resp}")
   }
   out
 }
@@ -200,18 +234,17 @@ stan_response <- function(bterms, data) {
 # intercepts in ordinal models require special treatment
 # and must be present even when using non-linear predictors
 # thus the relevant Stan code cannot be part of 'stan_fe'
-stan_thres <- function(bterms, data, prior, ...) {
+stan_thres <- function(bterms, data, prior, normalize, ...) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   out <- list()
-  family <- bterms$family
-  if (!is_ordinal(family)) {
+  if (!is_ordinal(bterms)) {
     return(out)
   }
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
-  type <- str_if(has_ordered_thres(family), "ordered", "vector")
-  coef_type <- str_if(has_ordered_thres(family), "", "real")
+  type <- str_if(has_ordered_thres(bterms), "ordered", "vector")
+  coef_type <- str_if(has_ordered_thres(bterms), "", "real")
   gr <- grb <- ""
   groups <- get_thres_groups(bterms)
   if (has_thres_groups(bterms)) {
@@ -221,7 +254,7 @@ stan_thres <- function(bterms, data, prior, ...) {
   }
   if (fix_intercepts(bterms)) {
     # identify ordinal mixtures by fixing their thresholds to the same values
-    if (has_equidistant_thres(family)) {
+    if (has_equidistant_thres(bterms)) {
       stop2("Cannot use equidistant and fixed thresholds at the same time.")
     }
     # separate definition from computation to support fixed parameters
@@ -235,19 +268,19 @@ stan_thres <- function(bterms, data, prior, ...) {
       "  Intercept{p}{gr} = fixed_Intercept{resp}{gr};\n"
     )
   } else {
-    if (has_equidistant_thres(family)) {
+    if (has_equidistant_thres(bterms)) {
       bound <- subset2(prior, class = "delta", group = "", ls = px)$bound
       for (i in seq_along(groups)) {
         str_add_list(out) <- stan_prior(
           prior, class = "Intercept", group = groups[i], 
           type = "real", prefix = "first_",
           suffix = glue("{p}{gr[i]}"), px = px, 
-          comment = "first threshold"
+          comment = "first threshold", normalize = normalize
         )
         str_add_list(out) <- stan_prior(
           prior, class = "delta", group = groups[i], 
           type = glue("real{bound}"), px = px, suffix = gr[i], 
-          comment = "distance between thresholds"
+          comment = "distance between thresholds", normalize = normalize
         )
       }
       str_add(out$tpar_def) <- 
@@ -270,21 +303,38 @@ stan_thres <- function(bterms, data, prior, ...) {
           coef = get_thres(bterms, group = groups[i]), 
           type = glue("{type}[nthres{resp}{grb[i]}]"),
           coef_type = coef_type, px = px, suffix = glue("{p}{gr[i]}"),
-          comment = "temporary thresholds for centered predictors"
+          comment = "temporary thresholds for centered predictors",
+          normalize = normalize
         )
       }
     }
   }
+  stz <- ""
+  if (has_sum_to_zero_thres(bterms)) {
+    stz <- "_stz"
+    str_add(out$tpar_def) <- cglue(
+      "  vector[nthres{resp}{grb}] Intercept{p}_stz{gr};",
+      "  // sum-to-zero constraint thresholds\n"
+    )
+    str_add(out$tpar_comp) <- "  // compute sum-to-zero constraint thresholds\n"
+    str_add(out$tpar_comp) <- cglue(
+      "  Intercept{p}_stz{gr} = Intercept{p}{gr} - mean(Intercept{p}{gr});\n"
+    )
+  }
   if (has_thres_groups(bterms)) {
     # merge all group specific thresholds into one vector
-    str_add(out$model_def) <- glue(
-      "  vector[nmthres{resp}] merged_Intercept{p};  // merged thresholds\n"
+    str_add(out$tpar_def) <- glue(
+      "  vector[nmthres{resp}] merged_Intercept{p}{stz};  // merged thresholds\n"
     )
+    str_add(out$tpar_comp) <- "  // merge thresholds\n"
     grj <- seq_along(groups)
     grj <- glue("Kthres_start{resp}[{grj}]:Kthres_end{resp}[{grj}]")
-    str_add(out$model_comp_basic) <- cglue(
-      "  merged_Intercept{p}[{grj}] = Intercept{p}{gr};\n"
+    str_add(out$tpar_comp) <- cglue(
+      "  merged_Intercept{p}{stz}[{grj}] = Intercept{p}{stz}{gr};\n"
     )
+    str_add(out$pll_args) <- cglue(", vector merged_Intercept{p}{stz}")
+  } else {
+    str_add(out$pll_args) <- glue(", vector Intercept{p}{stz}")
   }
   sub_X_means <- ""
   if (stan_center_X(bterms) && length(all_terms(bterms$fe))) {
@@ -296,49 +346,58 @@ stan_thres <- function(bterms, data, prior, ...) {
   str_add(out$gen_def) <- "  // compute actual thresholds\n"
   str_add(out$gen_def) <- cglue(
     "  vector[nthres{resp}{grb}] b{p}_Intercept{gr}",  
-    " = Intercept{p}{gr}{sub_X_means};\n" 
+    " = Intercept{p}{stz}{gr}{sub_X_means};\n" 
   )
   out
 }
 
 # Stan code for the baseline functions of the Cox model
-stan_bhaz <- function(bterms, prior, ...) {
+stan_bhaz <- function(bterms, prior, threads, normalize, ...) {
   stopifnot(is.btl(bterms) || is.btnl(bterms))
   out <- list()
   if (!is_cox(bterms$family)) {
     return(out)
   }
+  lpdf <- stan_lpdf_name(normalize)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   resp <- usc(px$resp)
+  slice <- stan_slice(threads)
   str_add(out$data) <- glue(
     "  // data for flexible baseline functions\n",
     "  int Kbhaz{resp};  // number of basis functions\n",
     "  // design matrix of the baseline function\n",
     "  matrix[N{resp}, Kbhaz{resp}] Zbhaz{resp};\n",
     "  // design matrix of the cumulative baseline function\n",
-    "  matrix[N{resp}, Kbhaz{resp}] Zcbhaz{resp};\n"
+    "  matrix[N{resp}, Kbhaz{resp}] Zcbhaz{resp};\n",
+    "  // a-priori concentration vector of baseline coefficients\n",
+    "  vector<lower=0>[Kbhaz{resp}] con_sbhaz{resp};\n"
   )
-  str_add_list(out) <- stan_prior(
-    prior, class = "sbhaz", suffix = resp, px = px, 
-    type = glue("vector<lower=0>[Kbhaz{resp}]"),
-    comment = "baseline coefficients"
+  str_add(out$par) <- glue(
+    "  simplex[Kbhaz{resp}] sbhaz{resp};  // baseline coefficients\n"
+  )
+  str_add(out$prior) <- glue(
+    "  target += dirichlet_{lpdf}(sbhaz{resp} | con_sbhaz{resp});\n"
   )
   str_add(out$model_def) <- glue(
     "  // compute values of baseline function\n",
-    "  vector[N{resp}] bhaz{resp} = Zbhaz{resp} * sbhaz{resp};\n",
+    "  vector[N{resp}] bhaz{resp} = Zbhaz{resp}{slice} * sbhaz{resp};\n",
     "  // compute values of cumulative baseline function\n",
-    "  vector[N{resp}] cbhaz{resp} = Zcbhaz{resp} * sbhaz{resp};\n"
+    "  vector[N{resp}] cbhaz{resp} = Zcbhaz{resp}{slice} * sbhaz{resp};\n"
+  )
+  str_add(out$pll_args) <- glue(
+    ", data matrix Zbhaz{resp}, data matrix Zcbhaz{resp}, vector sbhaz{resp}"
   )
   out
 }
 
 # Stan code specific to mixture families
-stan_mixture <- function(bterms, data, prior) {
+stan_mixture <- function(bterms, data, prior, threads, normalize, ...) {
   out <- list()
   if (!is.mixfamily(bterms$family)) {
     return(out)
   }
+  lpdf <- stan_lpdf_name(normalize)
   px <- check_prefix(bterms)
   p <- usc(combine_prefix(px))
   nmix <- length(bterms$family$mix)
@@ -355,7 +414,7 @@ stan_mixture <- function(bterms, data, prior) {
     }
     missing_id <- setdiff(1:nmix, dpar_id(names(theta_pred)))
     str_add(out$model_def) <- glue(
-      "  vector[N{p}] theta{missing_id}{p} = rep_vector(0, N{p});\n",                   
+      "  vector[N{p}] theta{missing_id}{p} = rep_vector(0.0, N{p});\n",                   
       "  real log_sum_exp_theta;\n"      
     )
     sum_exp_theta <- glue("exp(theta{1:nmix}{p}[n])", collapse = " + ")
@@ -377,6 +436,7 @@ stan_mixture <- function(bterms, data, prior) {
     str_add(out$data) <- cglue(
       "  real<lower=0,upper=1> theta{1:nmix}{p};\n"
     )
+    str_add(out$pll_args) <- cglue(", real theta{1:nmix}{p}")
   } else {
     # estimate mixture proportions
     str_add(out$data) <- glue(
@@ -386,7 +446,7 @@ stan_mixture <- function(bterms, data, prior) {
       "  simplex[{nmix}] theta{p};  // mixing proportions\n"
     )
     str_add(out$prior) <- glue(
-      "  target += dirichlet_lpdf(theta{p} | con_theta{p});\n"                
+      "  target += dirichlet_{lpdf}(theta{p} | con_theta{p});\n"                
     )
     # separate definition from computation to support fixed parameters
     str_add(out$tpar_def) <- "  // mixing proportions\n"
@@ -396,6 +456,7 @@ stan_mixture <- function(bterms, data, prior) {
     str_add(out$tpar_comp) <- cglue(
       "  theta{1:nmix}{p} = theta{p}[{1:nmix}];\n"
     )
+    str_add(out$pll_args) <- cglue(", real theta{1:nmix}{p}")
   }
   if (order_intercepts(bterms)) {
     # identify mixtures by ordering the intercepts of their components
@@ -422,7 +483,8 @@ stan_mixture <- function(bterms, data, prior) {
         type = glue("{type}[nthres{p}{grb[i]}]"), 
         coef_type = coef_type, px = px, 
         prefix = "fixed_", suffix = glue("{p}{gr[i]}"),
-        comment = "thresholds fixed over mixture components"
+        comment = "thresholds fixed over mixture components",
+        normalize = normalize
       )
     }
   }
@@ -457,50 +519,95 @@ stan_ordinal_lpmf <- function(family, link) {
   )
   # define the function body
   if (family == "cumulative") {
-    str_add(out) <- glue(
-      "     int nthres = num_elements(thres);\n",
-      "     real p;\n",
-      "     if (y == 1) {{\n",
-      "       p = {ilink}({th(1)});\n",
-      "     }} else if (y == nthres + 1) {{\n",
-      "       p = 1 - {ilink}({th('nthres')});\n",
-      "     }} else {{\n",
-      "       p = {ilink}({th('y')}) -\n",
-      "           {ilink}({th('y - 1')});\n",
-      "     }}\n",
-      "     return log(p);\n",
-      "   }}\n"
-    )
+    if (ilink == "inv_logit") {
+      str_add(out) <- glue(
+        "     int nthres = num_elements(thres);\n",
+        "     if (y == 1) {{\n",
+        "       return log_inv_logit({th(1)});\n",
+        "     }} else if (y == nthres + 1) {{\n",
+        "       return log1m_inv_logit({th('nthres')});\n",
+        "     }} else {{\n",
+        # TODO: replace with log_inv_logit_diff once rstan >= 2.25
+        "       return log_diff_exp(\n",
+        "         log_inv_logit({th('y')}), \n",
+        "         log_inv_logit({th('y - 1')})\n",
+        "       );\n",
+        "     }}\n",
+        "   }}\n"
+      )
+    } else {
+      str_add(out) <- glue(
+        "     int nthres = num_elements(thres);\n",
+        "     real p;\n",
+        "     if (y == 1) {{\n",
+        "       p = {ilink}({th(1)});\n",
+        "     }} else if (y == nthres + 1) {{\n",
+        "       p = 1 - {ilink}({th('nthres')});\n",
+        "     }} else {{\n",
+        "       p = {ilink}({th('y')}) -\n",
+        "           {ilink}({th('y - 1')});\n",
+        "     }}\n",
+        "     return log(p);\n",
+        "   }}\n"
+      )
+    }
   } else if (family %in% c("sratio", "cratio")) {
-    sc <- str_if(family == "sratio", "1 - ")
+    if (ilink == "inv_cloglog") {
+      qk <- str_if(
+        family == "sratio", 
+        "-exp({th('k')})",
+        "log1m_exp(-exp({th('k')}))"
+      )
+    } else if (ilink == "inv_logit") {
+      qk <- str_if(
+        family == "sratio", 
+        "log1m_inv_logit({th('k')})",
+        "log_inv_logit({th('k')})"
+      )
+    } else if (ilink == "Phi") {
+      # TODO: replace with more stable std_normal_lcdf once rstan >= 2.25
+      qk <- str_if(
+        family == "sratio", 
+        "normal_lccdf({th('k')}|0,1)",
+        "normal_lcdf({th('k')}|0,1)"
+      )
+    } else if (ilink == "Phi_approx") {
+      qk <- str_if(
+        family == "sratio",
+        "log1m_inv_logit(0.07056 * pow({th('k')}, 3.0) + 1.5976 * {th('k')})",
+        "log_inv_logit(0.07056 * pow({th('k')}, 3.0) + 1.5976 * {th('k')})"
+      )
+    } else if (ilink == "inv_cauchit") {
+      qk <- str_if(
+        family == "sratio",
+        "cauchy_lccdf({th('k')}|0,1)",
+        "cauchy_lcdf({th('k')}|0,1)"
+      )
+    }
+    qk <- glue(qk)
     str_add(out) <- glue(
       "     int nthres = num_elements(thres);\n",
       "     vector[nthres + 1] p;\n",
       "     vector[nthres] q;\n",
       "     int k = 1;\n",
       "     while (k <= min(y, nthres)) {{\n",
-      "       q[k] = {sc}{ilink}({th('k')});\n",
-      "       p[k] = 1 - q[k];\n",
-      "       for (kk in 1:(k - 1)) p[k] = p[k] * q[kk];\n", 
+      "       q[k] = {qk};\n",
+      "       p[k] = log1m_exp(q[k]);\n",
+      "       for (kk in 1:(k - 1)) p[k] = p[k] + q[kk];\n", 
       "       k += 1;\n",
       "     }}\n",
       "     if (y == nthres + 1) {{\n",
-      "       p[nthres + 1] = prod(q);\n",
+      "       p[nthres + 1] = sum(q);\n",
       "     }}\n",
-      "     return log(p[y]);\n",
+      "     return p[y];\n",
       "   }}\n"
     )
   } else if (family == "acat") {
     if (ilink == "inv_logit") {
       str_add(out) <- glue(
         "     int nthres = num_elements(thres);\n",
-        "     vector[nthres + 1] p;\n",
-        "     p[1] = 0.0;\n",
-        "     for (k in 1:(nthres)) {{\n",
-        "       p[k + 1] = p[k] + {th('k')};\n",
-        "     }}\n",
-        "     p = exp(p);\n",
-        "     return log(p[y] / sum(p));\n",
+        "     vector[nthres + 1] p = append_row(0, cumulative_sum(disc * (mu - thres)));\n",
+        "     return p[y] - log_sum_exp(p);\n",
         "   }}\n"
       )
     } else {
@@ -520,9 +627,8 @@ stan_ordinal_lpmf <- function(family, link) {
       )
     }
   }
-  # lpdf function for multiple merged thresholds
+  # lpmf function for multiple merged thresholds
   str_add(out) <- glue(
-    # TODO: include order_logistic_merged_lpdf
     "  /* {family}-{link} log-PDF for a single response and merged thresholds\n",
     "   * Args:\n",
     "   *   y: response category\n",
@@ -538,5 +644,23 @@ stan_ordinal_lpmf <- function(family, link) {
     "     return {family}_{link}_lpmf(y | mu, disc, thres[j[1]:j[2]]);\n",
     "   }}\n"
   )
+  if (family == "cumulative" && link == "logit") {
+    # use the more efficient 'ordered_logistic' built-in function 
+    str_add(out) <- glue(
+      "  /* ordered-logistic log-PDF for a single response and merged thresholds\n",
+      "   * Args:\n",
+      "   *   y: response category\n",
+      "   *   mu: latent mean parameter\n",
+      "   *   thres: vector of merged ordinal thresholds\n",
+      "   *   j: start and end index for the applid threshold within 'thres'\n",
+      "   * Returns:\n", 
+      "   *   a scalar to be added to the log posterior\n",
+      "   */\n",
+      "   real ordered_logistic_merged_lpmf(", 
+      "int y, real mu, vector thres, int[] j) {{\n",
+      "     return ordered_logistic_lpmf(y | mu, thres[j[1]:j[2]]);\n",
+      "   }}\n"
+    )
+  }
   out
 }
